@@ -7,6 +7,7 @@ import os
 import json
 import shutil
 import logging
+import urllib.request
 from interface import implements
 
 from xtlib import utils
@@ -42,17 +43,21 @@ class AzureML(BackendBase):
         self.username = username
 
         self.store = self.core.store
+        self.request = None
 
         # load azure libraries on demand
-        global Workspace, Experiment
+        global Workspace, Experiment, Run
         import azureml.core
-        from azureml.core import Experiment
+        from azureml.core import Experiment, Run
         from azureml.core.workspace import Workspace
 
         if disable_warnings:
             # turn off all azure warnings
             logging.getLogger("azureml").setLevel(logging.ERROR)
 
+    def get_name(self):
+        return "aml"
+
     def adjust_run_commands(self, job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, args):
         '''
         This method is called to allow the backend to inject needed shell commands before the user cmd.  At the
@@ -372,38 +377,6 @@ class AzureML(BackendBase):
 
         return sizes
 
-    # def make_distribution(self, dist_name, values):
-    #     from azureml.train.hyperdrive import choice, randint, uniform, normal, loguniform, lognormal
-    #     from azureml.train.hyperdrive import quniform, qnormal, qloguniform, qlognormal
-
-    #     if dist_name == "choice":
-    #         dist = choice(*values)
-    #     elif dist_name == "randint":
-    #         # optional limit
-    #         if not values or (len(values)==1 and not values[0]):
-    #             values = [2**32]
-    #         dist = randint(*values)
-    #     elif dist_name == "uniform":
-    #         dist = uniform(*values)
-    #     elif dist_name == "normal":
-    #         dist = normal(*values)
-    #     elif dist_name == "loguniform":
-    #         dist = loguniform(*values)
-    #     elif dist_name == "lognormal":
-    #         dist = lognormal(*values)
-    #     elif dist_name == "quniform":
-    #         dist = quniform(*values)
-    #     elif dist_name == "qnormal":
-    #         dist = qnormal(*values)
-    #     elif dist_name == "qloguniform":
-    #         dist = qloguniform(*values)
-    #     elif dist_name == "qlognormal":
-    #         dist = qlognormal(*values)
-    #     else:
-    #         errors.config_error("unknown distribution type: " + dist_name)
-
-    #     return dist
-        
     def build_hyperdrive_dict(self, hp_sets):
         hd = {}
 
@@ -777,7 +750,7 @@ class AzureML(BackendBase):
             # not using AML hyperdrive
             trainer = estimator
 
-        run_name, monitor_cmd, aml_run_name, aml_run_number = \
+        run_name, monitor_cmd, aml_run_name, aml_run_number, aml_run_id = \
             self.run_aml_job(job_id, workspace, aml_ws_name, trainer, experiment, xt_exper_name, aml_exper_name, compute_target, code_dir, first_run_name, 
                 box_name, node_index, repeat_count, fake_submit, args)
 
@@ -798,6 +771,7 @@ class AzureML(BackendBase):
             # we only have 1 run, so OK to hold info in flat dict here
             node_info["aml_exper_name"] = aml_exper_name
             node_info["aml_run_number"] = aml_run_number
+            node_info["aml_run_id"] = aml_run_id
             node_info["run_name"] = run_name
 
             # update mongo db info for run with cluster and service_job_id
@@ -834,7 +808,7 @@ class AzureML(BackendBase):
         username = args["username"]
         description = args["description"]
         aggregate_dest = args["aggregate_dest"]
-        monitor = args["monitor"]
+        jupyter_monitor = args["jupyter_monitor"]
 
         aml_run_name = aml_exper_name + ".{}".format(run_name)
 
@@ -848,13 +822,13 @@ class AzureML(BackendBase):
         #         box_name=box_name, app_name=None, repeat=repeat, is_parent=False, job_id=job_id, pool=compute_target, node_index=node_index,
         #         aggregate_dest=aggregate_dest, path=cwd, aml_run_id=aml_run_id)
 
-        if monitor:
+        if jupyter_monitor:
             fn = self.make_monitor_notebook(aml_ws_name, aml_run_name)
             dir = os.path.dirname(fn)
             #console.print("jupyter notebook written to: " + fn)
             monitor_cmd = "jupyter notebook --notebook-dir=" + dir
         
-        return run_name, monitor_cmd, aml_run_name, aml_run_number
+        return run_name, monitor_cmd, aml_run_name, aml_run_number, aml_run_id
 
     def get_client_cs(self, service_node_info):
         '''
@@ -881,3 +855,128 @@ class AzureML(BackendBase):
         dir_name = dataref.path_on_compute
         console.print("daatastore MOUNT dir_name=", dir_name)
         return dir_name
+
+    # API call
+    def read_log_file(self, service_node_info, log_name, start_offset=0, end_offset=None, 
+        encoding='utf-8', use_best_log=True):
+
+        run = self.get_node_run(service_node_info)
+
+        # aml bug workaround
+        run._output_logs_pattern = "azureml-logs/[\d]{2}.+\.txt"
+
+        # this is the style of log output we want, but as a pull-model
+        # stream log to console
+        #run.wait_for_completion(show_output=True)
+
+        new_text = None
+        node_status = "queued"
+        next_offset = None
+
+        # refresh run details (status and logs)
+        current_details = run.get_details() 
+        aml_status = current_details["status"] 
+        simple_status = self.get_simple_status(aml_status)
+
+        if use_best_log:
+            # list of currently available logs
+            available_logs = run._get_logs(current_details)
+
+            # get highest priority (most relevant to user) log name
+            next_log = Run._get_last_log_primary_instance(available_logs) if available_logs else None
+
+            if available_logs and log_name != next_log:
+                # switching logs, so reset offset and remember new name
+                start_offset = 0
+                log_name = next_log
+
+        if log_name:
+            # reuse request for better perf (hopefully)
+            url = current_details["logFiles"][log_name]
+
+            if not self.request:
+                self.request = urllib.request.Request(url)
+            elif self.request.full_url != url:
+                #self.request.close()
+                self.request = urllib.request.Request(url)
+
+            try:
+                with urllib.request.urlopen(self.request) as response:
+                    all_bytes = response.read()
+            except:
+                # treat any error as "log not yet available"
+                all_bytes = b""
+
+            if end_offset:
+                new_bytes = all_bytes[start_offset:1+end_offset]
+            else:
+                new_bytes = all_bytes[start_offset:]
+
+            next_offset = start_offset + len(new_bytes)
+            new_text = new_bytes.decode(encoding)
+
+        return {"new_text": new_text, "simple_status": simple_status, "log_name": log_name, "next_offset": next_offset, 
+            "service_status": aml_status}
+
+    # API call
+    def get_simple_status(self, status):
+        # translates an AML status to a simple status (queued, running, completed)
+
+        queued = ["NotStarted", "Starting", "Provisioning", "Preparing", "Queued"]
+        running = ["Running"]
+        completed = ["Finalizing", "CancelRequested", "Completed", "Failed", "Canceled", "NotResponding"]
+
+        if status in queued:
+            ss = "queued"
+        elif status in running:
+            ss = "running"
+        elif status in completed:
+            ss = "completed"
+        else:
+            errors.internal_error("unexpected Auzre ML status value: {}".format(status))
+
+        return ss
+
+    def get_node_run(self, service_node_info):
+        # create aml workspace
+        aml_ws_name = utils.safe_value(self.compute_def, "service")
+        ws = self.get_aml_ws(aml_ws_name)
+
+        # create aml experiment
+        aml_exper_name = service_node_info["aml_exper_name"]
+        experiment = Experiment(ws, name=aml_exper_name)
+
+        # create aml run
+        aml_run_id = service_node_info["aml_run_id"]
+        run = Run(experiment, aml_run_id)
+
+        return run
+
+    # API call
+    def cancel_job(self, service_job_info, service_info_by_node):
+        result_by_node = {}
+
+        for node_id, node_info in service_info_by_node.items():
+            result = self.cancel_node(node_info)
+            result_by_node[node_id] = result
+
+        return result_by_node
+
+    # API call
+    def cancel_node(self, service_node_info):
+        run = self.get_node_run(service_node_info)
+
+        service_status = run.get_status()
+        simple_status = self.get_simple_status(service_status)
+        cancelled = False
+
+        if simple_status != "completed":
+            
+            run.cancel()
+
+            service_status = run.get_status()
+            simple_status = self.get_simple_status(service_status)
+            cancelled = (service_status == "completed")     
+
+        result = {"cancelled": cancelled, "service_status": service_status, "simple_status": simple_status}
+        return result
