@@ -87,6 +87,9 @@ class AzureBatch(BackendBase):
         self.batch_client = None
         #console.print("blob_client=", blob_client)
 
+    def get_name(self):
+        return "batch"
+
     def adjust_run_commands(self, job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, args):
         '''
         This method is called to allow the backend to inject needed shell commands before the user cmd.  At the
@@ -901,7 +904,7 @@ class AzureBatch(BackendBase):
         
         try:
             status = "running"
-            task_counts = self.batch_client.self.get_task_counts(job_id)
+            task_counts = self.batch_client.job.get_task_counts(job_id)
             if task_counts.active:
                 # if any tasks are waiting for a node, consider the job status as allocating
                 status = "allocating"    
@@ -928,7 +931,7 @@ class AzureBatch(BackendBase):
             while True:   
                 
                 # get a dict of the stats for each task
-                task_counts = self.batch_client.self.get_task_counts(self.job_id)
+                task_counts = self.batch_client.job.get_task_counts(self.job_id)
 
                 elapsed = utils.elapsed_time(start)
                 console.print("waiting for queued job to start... (elapsed time: {}).format(elapsed)", end="\r")
@@ -939,13 +942,13 @@ class AzureBatch(BackendBase):
 
                 # check every .5 secs for keypress to be more responsive (but every 1 sec for task counts)
                 ch = checker.getch_nowait()
-                if ch == 27:
+                if ch == constants.ESCAPE:
                     detach_requested = True
                     break
                 time.sleep(.5)
 
                 ch = checker.getch_nowait()
-                if ch == 27:
+                if ch == constants.ESCAPE:
                     detach_requested = True
                     break
                 time.sleep(.5)
@@ -982,7 +985,7 @@ class AzureBatch(BackendBase):
                         text = data.decode("utf-8")
                         console.print(text)
 
-                    task_counts = self.batch_client.self.get_task_counts(self.job_id)
+                    task_counts = self.batch_client.job.get_task_counts(self.job_id)
                     if task_counts.running == 0:
                         console.print("<task terminated>")
                         break
@@ -993,6 +996,7 @@ class AzureBatch(BackendBase):
             else:
                 console.print("error - task has no node")
 
+
     def print_output_for_tasks(self):
         """Prints the stdout.txt file for each task in the self.
         """
@@ -1236,4 +1240,125 @@ class AzureBatch(BackendBase):
 
                 cancel_results_by_box.update(kr_by_box)
 
-        return cancel_results_by_box
\ No newline at end of file
+        return cancel_results_by_box
+
+    def get_node_task(self, service_node_info):
+        # get task_id
+        self.create_batch_client()
+
+        job_id = service_node_info["job_id"]
+        task_index = service_node_info["task_index"]
+
+        task_id = "task" + str(task_index)
+        task = self.batch_client.task.get(job_id, task_id)
+
+        return task
+
+    # BACKEND API
+    def get_node_status(self, service_node_info):
+
+        task = self.get_node_task(service_node_info)
+        return task.state.value
+
+    # BACKEND API
+    def read_log_file(self, service_node_info, log_name, start_offset=0, end_offset=None, 
+        encoding='utf-8', use_best_log=True):
+        '''
+        returns subset of specified log file as text.
+        '''
+        job_id = service_node_info["job_id"]
+        task = self.get_node_task(service_node_info)
+
+        if log_name is None or log_name in ["primary", "stdout", "stdout.txt"]:
+            file_path = "stdout.txt"
+        elif log_name in ["secondary", "error", "stderr", "stderr.txt"]:
+            file_path = "stderr.txt"
+        else:
+            file_path = log_name
+
+        new_text = None
+        batch_status = None
+        simple_status = None
+        next_offset = 0
+
+        if task:
+            node_info = task.node_info
+            batch_status = task.state.value
+            simple_status = self.get_simple_status(batch_status)
+
+            if not end_offset:
+                end_offset = 1024*1024*1024*16     # 16 GB should be big enough for a log file
+
+            gft_opts = batchmodels.FileGetFromTaskOptions(ocp_range='bytes={}-{}'.format(start_offset, end_offset))
+
+            try:
+                stream = self.batch_client.file.get_from_task(job_id, task.id, file_path=file_path, 
+                    file_get_from_task_options=gft_opts)
+
+                # put streamed output into "output"
+                output = io.BytesIO()
+                for data in stream:
+                    output.write(data)
+
+                new_bytes = output.getvalue()
+                new_text = new_bytes.decode(encoding)
+                next_offset = start_offset + len(new_bytes)
+
+            except (batchmodels.BatchErrorException) as ex:
+                # interpret this error as task has terminated
+                pass
+
+        return {"new_text": new_text, "simple_status": simple_status, "log_name": log_name, "next_offset": next_offset, 
+            "service_status": batch_status}
+
+    # API call
+    def get_simple_status(self, status):
+        # translates an BATCH status to a simple status (queued, running, completed)
+
+        queued = ["active", "preparing"]
+        running = ["running"]
+        completed = ["completed"]
+
+        if status in queued:
+            ss = "queued"
+        elif status in running:
+            ss = "running"
+        elif status in completed:
+            ss = "completed"
+        else:
+            errors.internal_error("unexpected Azure Batch status value: {}".format(status))
+
+        return ss
+
+    # API call
+    def cancel_job(self, service_job_info, service_info_by_node):
+        result_by_node = {}
+
+        for node_id, node_info in service_info_by_node.items():
+            result = self.cancel_node(node_info)
+            result_by_node[node_id] = result
+
+        return result_by_node
+
+    # API call
+    def cancel_node(self, service_node_info):
+        task = self.get_node_task(service_node_info)
+
+        service_status = task.state.value
+        simple_status = self.get_simple_status(service_status)
+        cancelled = False
+
+        if simple_status != "completed":
+            
+            job_id = service_node_info["job_id"]
+            self.batch_client.task.terminate(job_id, task.id)
+
+            # refresh the task for updated status
+            task = self.get_node_task(service_node_info)
+
+            service_status = task.state.value
+            simple_status = self.get_simple_status(service_status)
+            cancelled = (service_status == "completed")     
+
+        result = {"cancelled": cancelled, "service_status": service_status, "simple_status": simple_status}
+        return result
