@@ -17,6 +17,7 @@ from xtlib.helpers.feedbackParts import feedback as fb
 from .backend_base import BackendBase
 from .backend_interface import BackendInterface
 
+from xtlib import qfe
 from xtlib import utils
 from xtlib import errors
 from xtlib import pc_utils
@@ -40,9 +41,11 @@ class Philly(BackendBase):
         self.username = username
         self.store = self.core.store
 
-        if arg_dict:
-            self.cluster = arg_dict["cluster"]
-            self.vc = arg_dict["vc"]
+        self.cluster = utils.safe_value(arg_dict, "cluster")
+        self.vc = utils.safe_value(arg_dict, "vc")
+        
+    def get_name(self):
+        return "philly"
 
     def adjust_run_commands(self, job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, args):
         '''
@@ -90,6 +93,7 @@ class Philly(BackendBase):
         '''
         return True
         
+        
     def print_jobs(self, result, title, prop, title_prefix, columns):
         console.print("{}, {}:".format(title_prefix, title))
         jobs = result[prop]
@@ -136,6 +140,7 @@ class Philly(BackendBase):
         if "completed" in stage_flags:
             self.print_jobs(result, "COMPLETED", "finishedJobs", title, columns)
 
+
     def build_env_vars(self, user_env_vars, workspace, experiment, run_name, use_controller, node_id, box_secret, args):
 
         # start by copying job-common env vars
@@ -322,13 +327,17 @@ class Philly(BackendBase):
         Note: backend submitter functions are responsible for:
             - submitting the job (for each node, queue runs for that node)
             - return service job id's
+
+        TODO: eliminate compute_def argument on submit_job, since it has already been passed to the CTR.
         '''
         tmp_files = []
 
         # now, submit runs by node (a separate philly submit for each node)
         service_info_by_nodes = {}
-        cluster = compute_def["cluster"]
-        vc = compute_def["vc"]
+
+        cluster = utils.safe_value(compute_def, "cluster")
+        vc = utils.safe_value(compute_def, "vc")
+        self.username = args["username"]
 
         fb.feedback("submitting to philly.{}.{}: ".format(cluster, vc), add_seperator=False)
 
@@ -353,30 +362,36 @@ class Philly(BackendBase):
         return service_job_info, service_info_by_nodes
 
     def build_curl(self, cmd):
-        fn_cert =  file_utils.get_xtlib_dir() + "/controller_scripts/philly-cert.crt"
+        fn_cert =  file_utils.get_xtlib_dir() + "/public_certs/philly-cert.crt"
         use_cert = True
         cert_cmd = '--cacert "{}"'.format(fn_cert) if use_cert else "-k"
+        explicit = qfe.get_explicit_options()
 
-        if pc_utils.is_windows():
-            # for windows, we use built-in authentication
-            prefix = 'curl -s {} --ntlm --user : '.format(cert_cmd)
-        else:
-            # for linux, see if user provided a password file
-            fn_pass = os.path.realpath(os.path.expanduser("~/.xt/curl_config.txt"))
-            if os.path.exists(fn_pass):
-                # pw contains both username and pw
-                prefix = "curl {} -s -K '{}' --ntlm  : ".format(cert_cmd, fn_pass)
+        fn_pass = os.path.realpath(os.path.expanduser("~/.xt/curl_config.txt"))
+        if os.path.exists(fn_pass):
+            # FIRST priority: user provided a CURL CONFIG file
+            # assume file contains both username and pw
+            prefix = "curl {} -s -K '{}' --ntlm  : ".format(cert_cmd, fn_pass)
+        elif "username" in explicit or not pc_utils.is_windows():
+            # SECOND priority: user provided an explicit username or on LINUX
+
+            # we stop feedback since user will enter text (username/password)
+            fb.stop_feedback()
+
+            # we are about to run our curl command, which will prompt user for their password
+            console.print("\nTo skip this step, put '-u user:pw' in: {}".format(fn_pass))
+
+            if "username" in explicit:
+                username = self.username
             else:
-                fb.stop_feedback()
-
-                # we are about to run our curl command, which will prompt user for their password
-                console.print("\nTo skip this step, put '-u user:pw' in: {}".format(fn_pass))
-
                 # we cannot rely on self.username to be the user's username on Philly 
                 username = input("Philly username: ")
 
-                # provide username and let user enter password
-                prefix = "curl {} -su '{}' --ntlm  : ".format(cert_cmd, username)
+            # provide username and let user enter password
+            prefix = "curl {} -su '{}' --ntlm  : ".format(cert_cmd, username)
+        else:
+            # THIRD priority: use of windows, we use built-in authentication
+            prefix = 'curl -s {} --ntlm --user : '.format(cert_cmd)
 
         console.diag("curl cmd: {}".format(prefix + cmd))
         return prefix + cmd
@@ -428,8 +443,8 @@ class Philly(BackendBase):
             wrapped_cmds.append(wrapped_cmd)
             run_names.append(run_name)
 
-        vc = utils.safe_value(compute_def, "vc")            
         cluster = utils.safe_value(compute_def, "cluster")  
+        vc = utils.safe_value(compute_def, "vc")            
         sku = utils.safe_value(compute_def, "sku").upper()  
         queue = utils.safe_value(compute_def, "queue")      
 
@@ -519,7 +534,7 @@ class Philly(BackendBase):
         if fake_submit:
             service_job_id = "fake_service_id"
         else:
-            fn = file_utils.fix_slashes(fn, force_linux=True)
+            fn = file_utils.fix_slashes(fn, is_linux=True)
             #curl_cmd = 'curl -k --ntlm --user : -X POST -H "Content-Type: application/json" --data @{} https://philly/api/jobs > {} 2>&1'.format(fn, fn_log)
             curl_cmd = self.build_curl('-X POST -H "Content-Type: application/json" --data @{} https://philly/api/jobs'.format(fn))
 
@@ -558,40 +573,53 @@ class Philly(BackendBase):
         parts = shlex.split(curl_cmd)
         
         # don't redirect errors because that is where PASSWORD PROMPT appears
-        run_result = subprocess.run(parts, stdout=subprocess.PIPE)  # , stderr=subprocess.PIPE)
-        result = run_result.stdout.decode('utf-8')
+        run_result = subprocess.run(parts, stdout=subprocess.PIPE) 
+        text_result = run_result.stdout.decode('utf-8')
+        returncode =  run_result.returncode
 
-        if not result:
-            curl_errors = {6: "Couldn't resolve host"}
-            returncode =  run_result.returncode
+        if returncode:      # not text_result:
             msg = "result.returncode=" + str(returncode)
+            curl_errors = {6: "Couldn't resolve host"}
+
             if returncode in curl_errors:
+                # errors without msgs
                 msg = curl_errors[returncode] + " ({})".format(msg)
 
             errors.service_error("Unexpected Philly error: {}".format(msg))
 
-        if parse_result:
-            console.diag("run_curl_cmd: result={}".format(result))
+        console.diag("run_curl_cmd: result={}".format(text_result))
 
-            result = json.loads(result)    
-        return result
+        # result_text is usually either HTML or JSON
+        if parse_result and text_result.startswith("{"):
 
-    def view_console(self, job_id, job_info, info_verb, parse_result=True, extra=""):
+            try:
+                # try to parse as JSON
+                text_result = json.loads(text_result)    
+                if "Message" in text_result:
+                    text_result = text_result["Message"]
+            except:
+                pass
+
+        return text_result
+
+    def view_console(self, job_id, job_info, info_verb="log", parse_result=True, extra=""):
         '''get snapshot of console output for specified job'''
-        pool_info = job_info["pool_info"]
-        cluster = pool_info["cluster"]
-        vc = pool_info["vc"]
-
         # TODO: get this from run
         node_index = 0
+        node_id = utils.node_id(node_index)
 
         service_info_by_node = job_info["service_info_by_node"]  
-        node_info = service_info_by_node[node_index]
+        node_info = service_info_by_node[node_id]
         philly_job_id = node_info["service_id"]  
 
-        curl_cmd = self.build_curl('"https://philly/api/{}?clusterId={}&vcId={}&jobId={}&logType=stdout&logRev=latest&content=full&jobType=ns{}"'.format(info_verb, cluster, vc, philly_job_id, extra))
+        cluster = utils.safe_value(node_info, "cluster")
+        vc = utils.safe_value(node_info, "vc")
 
-        result = self.run_curl_cmd(cur_cmd, parse_result)
+        log_type = "log"  # log or stdout
+        curl_cmd = self.build_curl('"https://philly/api/{}?clusterId={}&vcId={}&jobId={}&logType={}&logRev=latest&content=full&jobType=ns{}"' \
+            .format(info_verb, cluster, vc, philly_job_id, log_type, extra))
+
+        result = self.run_curl_cmd(curl_cmd, parse_result)
         return result, philly_job_id
 
     def list_jobs(self, cluster, vc, username, status=None, max_finished=None, stage_flags=""):
@@ -748,7 +776,112 @@ class Philly(BackendBase):
 
         return ssh_cmd
 
+    # BACKEND API
+    def get_node_status(self, service_node_info):
+        # https://philly/api/status?clusterId=gcr&vcId=pnrsy&jobId=application_1447977864059_0276&jobType=cntk&content=partial
 
+        cluster = service_node_info["cluster"]
+        vc = service_node_info["vc"]
+        philly_job_id = service_node_info["service_id"]
 
+        curl_cmd = self.build_curl('"https://philly/api/status?clusterId={}&vcId={}&jobId={}&content=partial"' \
+            .format(cluster, vc, philly_job_id))
 
+        result = self.run_curl_cmd(curl_cmd, parse_result=True)
+        status = result["status"]
+        return status
 
+    # BACKEND API
+    def read_log_file(self, service_node_info, log_name, start_offset=0, end_offset=None, 
+        encoding='utf-8', use_best_log=True):
+        '''
+        returns subset of specified log file as text.
+        '''
+        cluster = service_node_info["cluster"]
+        vc = service_node_info["vc"]
+        philly_job_id = service_node_info["service_id"]
+
+        log_type = "stdout"  # log or stdout
+        info_verb = "log"
+        extra = ""
+
+        philly_status = self.get_node_status(service_node_info)
+        simple_status = self.get_simple_status(philly_status)
+
+        curl_cmd = self.build_curl('"https://philly/api/{}?clusterId={}&vcId={}&jobId={}&logType={}&logRev=latest&content=full&jobType=ns{}"' \
+            .format(info_verb, cluster, vc, philly_job_id, log_type, extra))
+        
+        result = self.run_curl_cmd(curl_cmd, parse_result=True)
+        # if result == "\n":      
+        #     # try log
+        #     log_type = "log"
+
+        #     curl_cmd = self.build_curl('"https://philly/api/{}?clusterId={}&vcId={}&jobId={}&logType={}&logRev=latest&content=full&jobType=ns{}"' \
+        #         .format(info_verb, cluster, vc, philly_job_id, log_type, extra))
+
+        #     result = self.run_curl_cmd(curl_cmd, parse_result=True)
+
+        if len(result) > 1:
+            node_status = "running"
+
+        if end_offset:
+            new_text = result[start_offset:1+end_offset]
+        else:
+            new_text = result[start_offset:]
+
+        next_offset = start_offset + len(new_text)
+
+        return {"new_text": new_text, "simple_status": simple_status, "log_name": log_name, "next_offset": next_offset, 
+            "service_status": philly_status}
+
+    # API call
+    def get_simple_status(self, status):
+        # translates an Philly status to a simple status (queued, running, completed)
+
+        queued = ["Queued"]
+        running = ["Running"]
+        completed = ["Pass", "Failed", "Killed"]
+
+        if status in queued:
+            ss = "queued"
+        elif status in running:
+            ss = "running"
+        elif status in completed:
+            ss = "completed"
+        else:
+            errors.internal_error("unexpected Philly status value: {}".format(status))
+
+        return ss
+
+    # API call
+    def cancel_job(self, service_job_info, service_info_by_node):
+        result_by_node = {}
+
+        for node_id, node_info in service_info_by_node.items():
+            result = self.cancel_node(node_info)
+            result_by_node[node_id] = result
+
+        return result_by_node
+
+    # API call
+    def cancel_node(self, service_node_info):
+        cluster = service_node_info["cluster"]
+        vc = service_node_info["vc"]
+        philly_job_id = service_node_info["service_id"]
+
+        philly_status = self.get_node_status(service_node_info)
+        simple_status = self.get_simple_status(philly_status)
+        cancelled = False
+
+        if simple_status != "completed":
+            
+            abort_cmd = self.build_curl("https://philly/api/abort?clusterId={}&jobId={}".format(cluster, philly_job_id))
+            result =  self.run_curl_cmd(abort_cmd)
+            result = result[philly_job_id]
+
+            philly_status = result["status"]
+            simple_status = self.get_simple_status(philly_status)
+            cancelled = (philly_status == "Killed")     
+
+        result = {"cancelled": cancelled, "service_status": philly_status, "simple_status": simple_status}
+        return result
