@@ -4,8 +4,10 @@
 #
 # backend_pool.py: handles the platform submit/manage operations for service_types of pool and local
 import os
+import time
 import uuid
 import shutil
+import stat
 from interface import implements
 
 from xtlib import utils
@@ -22,6 +24,8 @@ from xtlib.xt_client import XTClient
 
 from xtlib.console import console
 from xtlib.helpers.feedbackParts import feedback as fb
+from xtlib.psm.local_psm_client import LocalPsmClient
+from xtlib.psm.remote_psm_client import RemotePsmClient
 
 from .backend_interface import BackendInterface
 from .backend_base import BackendBase
@@ -38,6 +42,9 @@ class PoolBackend(BackendBase):
         self.env_vars = None
         self.client = self.core.client
 
+    def get_name(self):
+        return "pool"
+
     def adjust_run_commands(self, job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, 
         args):
         '''
@@ -50,6 +57,7 @@ class PoolBackend(BackendBase):
         # local or POOL of vm's
         wrapped_parts = None     # we use same cmd on each box/job
         data_local = args["data_local"]
+        model_local = args["model_local"]
 
         for i, box_runs in enumerate(job_runs):
             # wrap the user commands in FIRST RUN of each box (apply data/model actions)
@@ -72,11 +80,18 @@ class PoolBackend(BackendBase):
 
                 # data_local overrides store_data_dir for LOCAL machine
                 if using_localhost and data_local:
-                    store_data_dir = os.path.expanduser(data_local)
+                    store_data_dir = os.path.join(os.path.expanduser(data_local), store_data_dir)
                     data_action = "use_local"
                     if not "data" in actions:
                         actions.append("data")
 
+                # model_local overrides store_model_dir for LOCAL machine
+                if using_localhost and model_local:
+                    store_model_dir = os.path.join(os.path.expanduser(model_local), store_model_dir)
+                    model_action = "use_local"
+                    if not "model" in actions:
+                        actions.append("model")
+
                 setup = self.config.get_setup_from_target_def(self.compute_def)
 
                 env_vars = self.get_env_vars_for_box(box_name, box_info, i, box_secret)
@@ -115,8 +130,8 @@ class PoolBackend(BackendBase):
                 box_addr = box_info.address
                 port = constants.CONTROLLER_PORT
 
-                if self.core.client.is_controller_running(box_name, box_addr, port):
-                    errors.config_error("XT controller already running on box: " + box_name)
+                # if self.core.client.is_controller_running(box_name, box_addr, port):
+                #     errors.config_error("XT controller already running on box: " + box_name)
 
             # second pass - transfer the main script to each box and start it
             for i, box_runs in enumerate(job_runs):
@@ -134,111 +149,161 @@ class PoolBackend(BackendBase):
 
     def run_job_on_box(self, job_id, run_data_list, box_index, box_info, app_info, pool_info,  
             resume_name=None, repeat=None, using_hp=None, exper_name=None, snapshot_dir=None, args=None):
-        '''
-        copy the startup script to the specified box (using SCP) and run it (using SSH)
-        '''
+
         box_name = box_info.box_name
-        workspace = args["workspace"]
-        #console.print("box_name=", box_name, ", box_index=", box_index)
-
         box_addr = box_info.address
-        is_local = box_addr == "localhost"
-
-        src_windows = pc_utils.is_windows()
-        dst_windows = box_info.box_os == "windows"
-
-        hold_open = args["hold"]
-        run_data = run_data_list[0]
-        run_name = run_data["run_name"]
-        ws = args["workspace"]
-
-        # kill any running instances of the controller on the box (those that didn't shut down cleanly)
-        self.client.cancel_thru_os(box_name, False)
-
-        # send over files to box
-        dest_dir = utils.get_controller_cwd(dst_windows, is_local=False)
-        fn_bootstrap = file_utils.path_join(dest_dir, os.path.basename(self.fn_wrapped))
-
-        visible = args["show_controller"]
-        hold_open = True
-
-        if not src_windows:
-            # current only supported for local Windows machines
-            visible = False
-            hold_open = False
-
-        if is_local:
-            # LOCAL box
-            # COPY the snapshot directory (localhost)
-            file_utils.copy_tree(snapshot_dir, dest_dir, omit_dirs="__pycache__")
-            fb.feedback("files copied to box")
-            fb.feedback("{}/{}".format(ws, run_name))
-
-            # start controller  (localhost)
-            fn_log = os.path.expanduser("~/.xt/controller.log")
-
-            if hold_open and visible:
-                os.system("start cmd /K " + fn_bootstrap)
-            else:            
-                process_utils.start_async_run_detached(fn_bootstrap, dest_dir, fn_log, visible=visible)
-
-        else:
-            # REMOTE box
-            # copy the .zip file using scp
-            src_dir = os.path.expanduser(constants.CWD_DIR) if src_windows else constants.CWD_DIR
-            fn_src = file_utils.path_join(src_dir, constants.CODE_ZIP_FN)
-            fn_dest = file_utils.path_join(dest_dir, constants.CODE_ZIP_FN)
-
-            # create the directory on the pool machine thru SSH
-            cmd = "mkdir -p {}".format(dest_dir)
-            err_code2, output2 = process_utils.sync_run_ssh(self, box_addr, cmd)
-
-            # run the COPY thru SCP
-            process_utils.scp_copy_file_to_box(self, box_addr, fn_src, fn_dest)
-
-            # unzip the file
-            # 1. we need the remote box expanded fn_dest
-            err_code, output = process_utils.sync_run_ssh(self, box_addr, "ls " + fn_dest)
-            full_fn_dest = output.strip()
-
-            cmd = ''' python -c \\"import zipfile;  zip = zipfile.ZipFile('{}', 'r'); zip.extractall('{}')\\" '''.format(full_fn_dest, os.path.dirname(full_fn_dest))
-            err_code2, output2 = process_utils.sync_run_ssh(self, box_addr, cmd)
-
-            fb.feedback("files copied to box")
-            fb.feedback("{}/{}".format(ws, run_name))
-
-            # start the run
-            run_in_cmd_window = visible
-
-            if run_in_cmd_window:
-                # for debugging, open a new cmd window and watch real-time output from remote box
-                cmd = "bash --login " + fn_bootstrap
-                ssh_cmd = process_utils.make_ssh_cmd(box_addr, cmd)
-
-                local_cwd = utils.get_controller_cwd(True, is_local=True)
-                fn_ssh = os.path.join(local_cwd, "run_ssh.bat")
-                fn_log = os.path.join(local_cwd, "run_ssh.log")
-                file_utils.write_text_file(fn_ssh, ssh_cmd)
-
-                if hold_open and visible:
-                    os.system("start cmd /K " + fn_ssh)
-                else:
-                    process_utils.start_async_run_detached(cmd, local_cwd, fn_log, visible=True)
-            else:
-                # normal run 
-                #cmd = "bash --login " + fn_bootstrap
-                cmd = ' "nohup bash --login {} </dev/null   > ~/.xt/controller.log 2>&1 &" '.format(fn_bootstrap) 
-
-                process_utils.sync_run_ssh(self, box_addr, cmd)
-
-        fb.feedback("submitted", is_final=True)  
+        box_os = box_info.box_os
+        is_box_windows = (box_os == "windows")
         
-        monitor_url = None
+        if pc_utils.is_localhost(box_addr=box_addr):
+            psm_client = LocalPsmClient()
+        else:
+            psm_client = RemotePsmClient(box_addr, is_box_windows)
 
-        service_node_info = {"job_id": job_id, "node_index": box_index, "box_name": box_name, "monitor_url": monitor_url}
+        psm_client.start_psm_if_needed()
+        #print("psm created for box: " + box_addr)
+
+        team = self.config.get("general", "xt-team-name")
+        node_id = utils.node_id(box_index)
+
+        cwd_dir = os.path.expanduser(constants.CWD_DIR)
+        fn_src_zip = file_utils.path_join(cwd_dir, constants.CODE_ZIP_FN)
+
+        fn_entry = psm_client.enqueue(team, job_id, node_id, fn_src_zip)
+
+        service_node_info = {"fn_entry": fn_entry, "box_addr": box_addr, "box_os": box_os, 
+            "box_name": box_name, "job_id": job_id}
+
+        fb.feedback("submitted", is_final=True) 
 
         return service_node_info
 
+    # def old_run_job_on_box(self, job_id, run_data_list, box_index, box_info, app_info, pool_info,  
+    #         resume_name=None, repeat=None, using_hp=None, exper_name=None, snapshot_dir=None, args=None):
+    #     '''
+    #     copy the startup script to the specified box (using SCP) and run it (using SSH)
+    #     '''
+    #     box_name = box_info.box_name
+    #     workspace = args["workspace"]
+    #     #console.print("box_name=", box_name, ", box_index=", box_index)
+
+    #     box_addr = box_info.address
+    #     is_local = box_addr == "localhost"
+
+    #     src_windows = pc_utils.is_windows()
+    #     dst_windows = box_info.box_os == "windows"
+
+    #     hold_open = args["hold"]
+    #     run_data = run_data_list[0]
+    #     run_name = run_data["run_name"]
+    #     ws = args["workspace"]
+
+    #     # kill any running instances of the controller on the box (those that didn't shut down cleanly)
+    #     self.client.cancel_thru_os(box_name, False)
+
+    #     # send over files to box
+    #     dest_dir = utils.get_controller_cwd(dst_windows, is_local=False)
+    #     fn_bootstrap = file_utils.path_join(dest_dir, os.path.basename(self.fn_wrapped))
+
+    #     visible = args["show_controller"]
+    #     hold_open = True
+
+    #     if is_local:
+    #         # create the directory on the local machine
+    #         os.makedirs(os.path.expanduser(dest_dir), exist_ok=True)
+
+    #         # LOCAL box
+    #         # COPY the snapshot directory (localhost)
+    #         dest_dir = os.path.expanduser(dest_dir)
+    #         file_utils.copy_tree(snapshot_dir, dest_dir, omit_dirs="__pycache__")
+    #         fb.feedback("files copied to box")
+    #         fb.feedback("{}/{}".format(ws, run_name))
+
+    #         # Make the wrapper script executable
+    #         if not src_windows:
+    #             wrapper_cmd = "{}/{}".format(
+    #                 dest_dir,
+    #                 constants.FN_WRAPPED_CMDS)
+    #             st = os.stat(wrapper_cmd)
+    #             os.chmod(wrapper_cmd, st.st_mode | stat.S_IEXEC)
+
+    #         # start controller  (localhost)
+    #         fn_log = os.path.expanduser("~/.xt/controller.log")
+    #         fn_bootstrap = os.path.expanduser(fn_bootstrap)
+
+    #         if hold_open and visible:
+    #             if src_windows:
+    #                 os.system("start cmd /K " + fn_bootstrap)
+    #             else:
+    #                 process_utils.start_async_run_detached(fn_bootstrap, os.path.expanduser(dest_dir), fn_log, visible=visible)
+    #         else:            
+    #             process_utils.start_async_run_detached(fn_bootstrap, os.path.expanduser(dest_dir), fn_log, visible=visible)
+
+    #     else:
+    #         # REMOTE box
+    #         # copy the .zip file using scp
+    #         src_dir = os.path.expanduser(constants.CWD_DIR)
+    #         fn_src = file_utils.path_join(src_dir, constants.CODE_ZIP_FN)
+    #         fn_src = os.path.expanduser(fn_src)
+
+    #         fn_dest = file_utils.path_join(dest_dir, constants.CODE_ZIP_FN)
+
+    #         # create the directory on the pool machine thru SSH
+    #         cmd = "mkdir -p {}".format(dest_dir)
+    #         fb.feedback("[md")
+    #         err_code2, output2 = process_utils.sync_run_ssh(self, box_addr, cmd)
+
+    #         # run the COPY thru SCP
+    #         fb.feedback("cp")
+    #         process_utils.scp_copy_file_to_box(self, box_addr, fn_src, fn_dest)
+
+    #         # unzip the file
+    #         # 1. we need the remote box expanded fn_dest
+    #         err_code, output = process_utils.sync_run_ssh(self, box_addr, "ls " + fn_dest)
+    #         full_fn_dest = output.strip().split("\n")[-1]
+
+    #         #cmd = ''' python -c \\"import zipfile;  zip = zipfile.ZipFile('{}', 'r'); zip.extractall('{}')\\" '''.format(full_fn_dest, os.path.dirname(full_fn_dest))
+    #         py_cmd = "import zipfile; zip = zipfile.ZipFile('{}', 'r'); zip.extractall('{}')".format(full_fn_dest, os.path.dirname(full_fn_dest))
+    #         ssh_cmd = 'python -c "{}"'.format(py_cmd)
+    #         fb.feedback("unzip]")
+    #         err_code2, output2 = process_utils.sync_run_ssh(self, box_addr, ssh_cmd)
+
+    #         fb.feedback("files copied to box")
+    #         fb.feedback("{}/{}".format(ws, run_name))
+
+    #         # start the run
+    #         run_in_cmd_window = visible
+
+    #         if run_in_cmd_window:
+    #             # for debugging, open a new cmd window and watch real-time output from remote box
+    #             cmd = "bash --login " + fn_bootstrap
+    #             ssh_parts = process_utils.make_ssh_cmd_parts(box_addr, cmd)
+    #             ssh_cmd = " ".join(ssh_parts)
+
+    #             local_cwd = utils.get_controller_cwd(True, is_local=True)
+    #             fn_ssh = os.path.join(local_cwd, "run_ssh.bat")
+    #             fn_log = os.path.join(local_cwd, "run_ssh.log")
+    #             file_utils.write_text_file(fn_ssh, ssh_cmd)
+
+    #             if hold_open and visible:
+    #                 os.system("start cmd /K " + fn_ssh)
+    #             else:
+    #                 process_utils.start_async_run_detached(fn_ssh, local_cwd, fn_log, visible=True)
+    #         else:
+    #             # normal run 
+    #             #cmd = "bash --login " + fn_bootstrap
+    #             cmd = 'nohup bash --login {} </dev/null > ~/.xt/controller.log 2>&1 &'.format(fn_bootstrap) 
+
+    #             process_utils.sync_run_ssh(self, box_addr, cmd)
+
+    #     fb.feedback("submitted", is_final=True)  
+        
+    #     monitor_url = None
+
+    #     service_node_info = {"job_id": job_id, "node_index": box_index, "box_name": box_name, "monitor_url": monitor_url}
+
+    #     return service_node_info
+
     def get_env_vars_for_box(self, box_name, box_info, nodex_index, box_secret):
         #box_secrets.set_secret(box_name, box_secret)
 
@@ -381,3 +446,112 @@ class PoolBackend(BackendBase):
         cancel_results =  self.cancel_runs_by_property("username", self.username, box_name)
         return cancel_results
 
+    def get_psm_client(self, service_node_info):
+        box_os = service_node_info["box_os"]
+        box_addr = service_node_info["box_addr"]
+        is_box_windows = (box_os == "windows")
+
+        if pc_utils.is_localhost(box_addr=box_addr):
+            psm_client = LocalPsmClient()
+        else:
+            psm_client = RemotePsmClient(box_addr, is_box_windows)
+
+        return psm_client
+
+    # BACKEND API
+    def read_log_file(self, service_node_info, log_name, start_offset=0, end_offset=None, 
+        encoding='utf-8', use_best_log=True):
+        '''
+        returns subset of specified log file as text.
+        '''
+        job_id = service_node_info["job_id"]
+        fn_entry = service_node_info["fn_entry"]
+
+        psm_client = self.get_psm_client(service_node_info)
+
+        next_offset = 0
+        new_text = None
+
+        # read LOG FILE
+        #start = time.time()
+        new_bytes = psm_client.read_log_file(fn_entry, start_offset, end_offset)
+        next_offset = start_offset + len(new_bytes)
+        #utils.report_elapsed(start, "read log")
+
+        new_text = new_bytes.decode(encoding)
+
+        if new_text:
+            service_status = "running"
+        else:
+            # refresh STATUS (takes 1.5, 4.5, or 6 secs)
+            #start = time.time()
+            service_status = psm_client.get_status(fn_entry)
+            #utils.report_elapsed(start, "status")
+
+        simple_status = self.get_simple_status(service_status)
+
+        return {"new_text": new_text, "simple_status": simple_status, "log_name": log_name, "next_offset": next_offset, 
+            "service_status": service_status}
+
+    # API call
+    def get_simple_status(self, status):
+        # translates an BATCH status to a simple status (queued, running, completed)
+
+        queued = ["queued"]
+        running = ["running"]
+        completed = ["completed"]
+
+        if status in queued:
+            ss = "queued"
+        elif status in running:
+            ss = "running"
+        elif status in completed:
+            ss = "completed"
+        else:
+            errors.internal_error("unexpected Pool status value: {}".format(status))
+
+        return ss
+
+    # API call
+    def cancel_job(self, service_job_info, service_info_by_node):
+        result_by_node = {}
+
+        for node_id, node_info in service_info_by_node.items():
+            result = self.cancel_node(node_info)
+            result_by_node[node_id] = result
+
+        return result_by_node
+
+    # API call
+    def cancel_node(self, service_node_info):
+
+        psm_client = self.get_psm_client(service_node_info)
+        fn_entry = service_node_info["fn_entry"]
+
+        service_status = psm_client.get_status(fn_entry)
+        simple_status = self.get_simple_status(service_status)
+        cancelled = False
+
+        if simple_status != "completed":
+            
+            cancelled, service_status = psm_client.cancel(fn_entry)
+            simple_status = self.get_simple_status(service_status)
+
+        result = {"cancelled": cancelled, "service_status": service_status, "simple_status": simple_status}
+        return result
+
+    # API call
+    def get_service_queue_entries(self, service_node_info):
+
+        psm_client = self.get_psm_client(service_node_info)
+        entries, current = psm_client.enum_queue()
+
+        queue = []
+        
+        if entries:
+            queue = [{"name": entry, "current": False} for entry in entries]
+
+        if current:
+            queue.insert(0, {"name": current, "current": True})
+
+        return queue
\ No newline at end of file
