@@ -57,31 +57,32 @@ class CmdCore():
                 errors.user_exit("existing XT keypair found (use --overwrite to force a new keypair to be generated)")
 
         # call ssh-keygen to do the GENERATION
-        cmd = 'ssh-keygen -q -f "' + fn + '"'
-        #console.print("keygen: cmd=", cmd)
-        exit_code, output = process_utils.sync_run(cmd)
+        cmd_parts = ['ssh-keygen', '-q', '-f', fn]
+        exit_code, output = process_utils.sync_run(cmd_parts)
         if exit_code:
             console.print(output)
             return False
 
         # ensure ssh-agent is ENABLED
-        cmd = "sc config ssh-agent start= demand"
-        exit_code, output = process_utils.sync_run(cmd)
+        #cmd = "sc config ssh-agent start=demand"
+        cmd_parts = ["sc", "config", "ssh-agent", "start=demand"]
+        exit_code, output = process_utils.sync_run(cmd_parts, shell=True)
         if exit_code:
+            # this call will FAIL unless running in a ADMINISTRATOR command window
             console.print(output)
             return False
 
         # ensure ssh-agent is RUNNING
-        cmd = "ssh-agent s"
-        exit_code, output = process_utils.sync_run(cmd)
+        cmd_parts = ["ssh-agent", "s"]
+        exit_code, output = process_utils.sync_run(cmd_parts)
         if exit_code:
             console.print(output)
             return False
 
         # finally, ADD the generated key to the ssh repository
-        cmd = 'ssh-add "{}"'.format(fn)
+        cmd_parts = ['ssh-add', fn]
         #console.print("keygen: cmd=", cmd)
-        exit_code, output = process_utils.sync_run(cmd)
+        exit_code, output = process_utils.sync_run(cmd_parts)
         if exit_code:
             console.print(output)
             return False
@@ -106,11 +107,12 @@ class CmdCore():
             temp_key_fn = "/tmp/temp_key_file"
 
         # NOTE: the "-o IdentitiesOnly=yes" option of is used to prevent the "too many authentication errors" problem 
-        cmd = 'scp -o IdentitiesOnly=yes "{}" {}:{}'.format(fn_local_key, box_addr, temp_key_fn)
-        console.diag("  copying key file to box: cmd=" + cmd)
+        #cmd = 'scp -o IdentitiesOnly=yes "{}" {}:{}'.format(fn_local_key, box_addr, temp_key_fn)
+        cmd_parts = ["scp", "-o", "IdentitiesOnly=yes", fn_local_key, "{}:{}".format(box_addr, temp_key_fn)]
+        console.diag("  copying key file to box: cmd={}".format(cmd_parts))
 
         # SCP COPY
-        exit_code, output = process_utils.sync_run(cmd)
+        exit_code, output = process_utils.sync_run(cmd_parts)
         if exit_code:
             console.print(output)
             return False
@@ -136,11 +138,12 @@ class CmdCore():
             cmdline = ";".join(cmds)
 
         # NOTE: the "-o IdentitiesOnly=yes" option of is used to prevent the "too many authentication errors" problem 
-        cmd = 'ssh -o IdentitiesOnly=yes {} "{}"'.format(box_addr, cmdline)
-        console.diag("  running cmds on box=" + cmd)
+        #cmd = 'ssh -o IdentitiesOnly=yes {} "{}"'.format(box_addr, cmdline)
+        cmd_parts = ['ssh', '-o', 'IdentitiesOnly=yes', box_addr, cmdline]
+        console.diag("  running cmds on box={}".format(cmd_parts))
 
         # SSH COMMANDS
-        exit_code, output = process_utils.sync_run(cmd)
+        exit_code, output = process_utils.sync_run(cmd_parts)
         if exit_code:
             console.print(output)
             return False
@@ -525,6 +528,9 @@ class CmdCore():
         with open(fn_context, "wt") as tfile:
             tfile.write(text)
 
+        # also write the MRC file to the job store (to support wrapup of runs after job is cancelled)
+        self.store.create_job_file(job_id, constants.FN_MULTI_RUN_CONTEXT, text)
+
         # write 2rd file to SNAPSHOT
         fn_script = snapshot_dir + "/" + constants.PY_RUN_CONTROLLER
         is_aml = (service_type == "aml")
@@ -593,6 +599,10 @@ class CmdCore():
         return copied_files
 
     def create_backend(self, compute, username=None, cluster=None, vc=None):
+        '''
+        TODO: replace "cluster" and "vc" arguments with regularized 
+        "compute_def" argument.
+        '''
 
         compute_def = self.config.get_compute_def(compute)
         service = compute_def["service"]
@@ -618,6 +628,10 @@ class CmdCore():
 
         # instantiate the associated backend class
         arg_dict = {"cluster": cluster, "vc": vc}
+
+        '''
+        TODO: eliminate "arg_dict" argument for backend CTR by regularizing all options into "compute_def".
+        '''
         backend = backend_ctr(compute=compute, compute_def=compute_def, core=self, config=self.config, username=username, arg_dict=arg_dict)
         return backend
 
