@@ -4,7 +4,7 @@
 #
 # constants.py: constant strings and values share among multiple modules
 # sync this version with setup.py, CHANGELOG.md
-BUILD = "version: 0.0.190, build: Apr-15-2020"
+BUILD = "version: 0.0.199, build: Apr-30-2020"
 
 INFO_CONTAINER = "xt-store-info"
 INFO_DIR = "__info__"
@@ -114,3 +114,14 @@ distribution_types = [
     "quniform", "qnormal", "qloguniform", "qlognormal", 
     ]
 
+ESCAPE = '\x1b'
+CONTROL_C = '\x03'
+
+# psm modules (keep manually in-sync with standalone psm.py)
+PSM_QUEUE = "~/.xt/psm_queue"
+CWD = "~/.xt/cwd"
+PSM = "psm.py"
+PSMLOG = "psm.log"
+PSM_LOGDIR = "~/.xt/psm_logs"
+CURRENT_RUNNING_ENTRY = "_current_running_entry_.txt"
+
