@@ -1453,7 +1453,8 @@ def debug_break():
     ptvsd.wait_for_attach()
     breakpoint()
 
-def run(concurrent=1, my_ip_addr=None, multi_run_context_fn=constants.FN_MULTI_RUN_CONTEXT, multi_run_hold_open=False, port=constants.CONTROLLER_PORT, is_aml=False):
+def run(concurrent=1, my_ip_addr=None, multi_run_context_fn=constants.FN_MULTI_RUN_CONTEXT, multi_run_hold_open=False, 
+        port=constants.CONTROLLER_PORT, is_aml=False):
     '''
     Runs the XT controller app - responsible for launch and control of all user ML apps for a
     local machine, remote machine, Azure VM, or Azure Batch VM.
@@ -1471,7 +1472,7 @@ def run(concurrent=1, my_ip_addr=None, multi_run_context_fn=constants.FN_MULTI_R
     #print_env_vars()
     
     box_secret = os.getenv("XT_BOX_SECRET")
-    console.print("box_secret: ", box_secret)
+    console.print("XT_BOX_SECRET: ", box_secret)
 
     # create the controller
     service = XTController(concurrent, my_ip_addr, multi_run_context_fn, multi_run_hold_open, port, is_aml)
