@@ -49,7 +49,7 @@ def parse_cmdline_args():
     parser.add_argument('--gpu', type=int, default=0, help='specify which gpu to use')
     parser.add_argument('--parallel', type=int, default=0, help='when specified, will do parallel training on all gpus')
     parser.add_argument('--distributed', type=int, default=0, help='when specified, will do distributed training on all nodes')
-    parser.add_argument('--data', type=str, default="~/.data", metavar='N', help='where to get/store MNIST data')
+    parser.add_argument('--data', type=str, default="data/mnist", metavar='N', help='where to get/store MNIST data')
     parser.add_argument('--search-api', type=float, default=0, help='should app call search API?')
     parser.add_argument('--raise-error', type=float, default=0, help='when =1, app will intentionaly raise error')
 
@@ -297,7 +297,7 @@ class Trainer():
         with open(self.fn_text_log, "a") as outfile:
             outfile.write(msg + "\n")
 
-    def log_stats(self, epoch, steps, data_len, loss, total_correct, total, model, device, 
+    def log_stats_and_test(self, epoch, steps, data_len, loss, total_correct, total, model, device, 
         checkpoint_freq, run, train_loss, train_acc, args):
 
         msg = 'Train Epoch: {} [{}/{} ({:.0f}%)]\tLoss: {:.6f}\tAcc: {:.6f}'.format(
@@ -310,12 +310,6 @@ class Trainer():
         # log to simple text logger
         self.text_log(msg)
 
-        test_loss, test_acc = self.test(args, model, device)
-        #print("test_loss=", test_loss, ", test_acc=", test_acc)
-
-        # time to checkpoint the model?
-        #print("epoch=", epoch, ", checkpoint_freq=", checkpoint_freq, "checkpoint_units=", checkpoint_units)
-
         if checkpoint_freq and run and run.store:
             if checkpoint_units == "epochs" and epoch % checkpoint_freq == 0:
                 cp_now = True
@@ -338,44 +332,47 @@ class Trainer():
 
             # log EVAL/TEST stats half as often
             if (epoch / args.log_interval ) % 2 == 0:
-                run.log_metrics({"epoch": epoch, "loss": test_loss, "acc": test_acc}, step_name="epoch", stage="test")
+                test_loss, test_acc = self.test_model_and_log_metrics(run, model, device, epoch, args)
 
-        # early stopping
-        if math.isnan(test_loss):
-            run.log_event("early_stopping", {"reason": "loss_is_nan"})
-            # exit without error
-            sys.exit(0)
+                # early stopping
+                if math.isnan(test_loss):
+                    run.log_event("early_stopping", {"reason": "loss_is_nan"})
+                    # exit without error
+                    sys.exit(0)
+
+    def test_model_and_log_metrics(self, run, model, device, epoch, args):
+        # TEST the model
+        test_loss, test_acc = self.test(args, model, device)
+        
+        # log TEST METRICS
+        #print("test_loss=", test_loss, ", test_acc=", test_acc)
+        run.log_metrics({"epoch": epoch, "loss": test_loss, "acc": test_acc}, step_name="epoch", stage="test")
+
+        return test_loss, test_acc
 
     def train_test_loop(self, run, model, device, optimizer, start_epoch, checkpoint_freq, 
-        test_only, args):
+        args):
 
         total_steps = 0
         start = time.time()
+        print("train_test_loop: start_epoch={} end_epoch={}\n".format(start_epoch, args.epochs + 1))
 
         for epoch in range(start_epoch, args.epochs + 1):
+
+            # train an epoch
+            train_loss, train_acc, steps, data_len,  loss, total_correct, total = \
+                self.train(args, model, device, optimizer, epoch)
+
+            total_steps += steps
+
+            if epoch % args.log_interval == 0:
+                elapsed = time.time() - start
+                #print("{} epoch(s) training took: {:.2f} secs".format(args.log_interval, elapsed))
+
+                self.log_stats_and_test(epoch, steps, data_len, loss, total_correct, total, model, device, 
+                    checkpoint_freq, run, train_loss, train_acc, args)
             
-
-            if test_only:
-                train_loss = 0
-                train_acc = 0
-            else:
-                # train an epoch
-                train_loss, train_acc, steps, data_len,  loss, total_correct, total = \
-                    self.train(args, model, device, optimizer, epoch)
-
-                total_steps += steps
-
-                if epoch % args.log_interval == 0:
-                    elapsed = time.time() - start
-                    print("{} epoch(s) training took: {:.2f} secs".format(args.log_interval, elapsed))
-
-                    self.log_stats(epoch, steps, data_len, loss, total_correct, total, model, device, 
-                        checkpoint_freq, run, train_loss, train_acc, args)
-                
-                    start = time.time()
-
-            if test_only:
-                break
+                start = time.time()
 
     def init_xt_run(self, logging, tb_path, args):
         # init xtlib
@@ -598,12 +595,12 @@ class Trainer():
             abc.foo = 1
 
         # log hyperparameters to xt
-        hp_dict = {"seed":args.seed, "batch-size": args.batch_size, "epochs": args.epochs, "lr": args.lr, 
-            "momentum": args.momentum, "channels1": args.channels1, "channels2": args.channels2, "kernel_size": args.kernel_size, 
-                "mlp-units": args.mlp_units, "weight-decay": args.weight_decay, "optimizer": args.optimizer, 
-                "mid-conv": args.mid_conv, "gpu": args.gpu, "log-interval": args.log_interval}
-
         if run:
+            hp_dict = {"seed":args.seed, "batch-size": args.batch_size, "epochs": args.epochs, "lr": args.lr, 
+                "momentum": args.momentum, "channels1": args.channels1, "channels2": args.channels2, "kernel_size": args.kernel_size, 
+                    "mlp-units": args.mlp_units, "weight-decay": args.weight_decay, "optimizer": args.optimizer, 
+                    "mid-conv": args.mid_conv, "gpu": args.gpu, "log-interval": args.log_interval}
+
             run.log_hparams(hp_dict)
 
         if args.cuda:
@@ -661,21 +658,23 @@ class Trainer():
                     checkpoint_freq = float(args.checkpoint)
                     checkpoint_units = "epochs"
 
-        model_dir = os.getenv("XT_MODEL_DIR", "models")
+        model_dir = os.getenv("XT_MODEL_DIR", "models/minimnist")
         fn_model = model_dir + "/mnist_cnn.pt"
         self.fn_text_log = mnt_output_dir + "/text_log.txt"
 
         if args.eval_model:
             # load model and evaluate it
             print("loading existing MODEL and evaluating it, fn=", fn_model)
+            exists = os.path.exists(fn_model)
+            print("model exists=", exists)
 
             model.load_state_dict(torch.load(fn_model))
+            print("model loaded!")
 
-            self.train_test_loop(run, model, device, optimizer, start_epoch, checkpoint_freq, 
-                test_only=True, args=args)
+            # just test model
+            self.test_model_and_log_metrics(run, model, device, epoch=1, args=args)
         else:
-            self.train_test_loop(run, model, device, optimizer, start_epoch, checkpoint_freq, 
-                test_only=False, args=args)
+            self.train_test_loop(run, model, device, optimizer, 1, checkpoint_freq, args=args)
 
         if (args.save_model):
             file_utils.ensure_dir_exists(model_dir)
