@@ -122,7 +122,7 @@ class XTConfig():
         
     def get_vault_key(self, name):
         self.create_vault_if_needed()
-        return self.vault.keys[name]
+        return self.vault.get_secret(name)
         
     def create_vault_if_needed(self):
         if not self.vault:
@@ -147,7 +147,7 @@ class XTConfig():
                 self.create_vault_if_needed()
 
             assert section and prop_name
-            text = self.vault.get_secret(section, prop_name)
+            text = self.vault.get_secret(section)
 
         if "$username" in text:
             ev_user = "USERNAME" if pc_utils.is_windows() else "USER"
@@ -184,7 +184,7 @@ class XTConfig():
         # read config file
         try:
             with open(fn, "rt") as file:
-                config = yaml.load(file, Loader=yaml.FullLoader)
+                config = yaml.safe_load(file)  # , Loader=yaml.FullLoader)
         except BaseException as ex:
             logger.exception("Error in read_config, ex={}".format(ex))
             raise Exception ("The config file '{}' is not valid YAML, error: {}".format(fn, ex))
@@ -366,11 +366,24 @@ class XTConfig():
         code_path = providers[name]
         return utils.get_class_ctr(code_path)
 
+    # def get_regularized_compute_def(self, compute, opt_names, explicit, args):
+    #     compute_def = self.get_compute_def(compute)  
+    #     if compute_def:
+    #         for name in opt_names:
+    #             under_name = name.replace("-", "_")
+
+    #             if name in explicit:
+    #                 sd[name] = args[under_name]
+    #             elif name in sd:
+    #                 args[under_name] = sd[name]
+
+    #     return compute_def
+
 # flat functions
 
 def load_yaml(fn): 
     with open(fn, "rt") as file:
-        data = yaml.load(file, Loader=yaml.FullLoader)
+        data = yaml.safe_load(file) #, Loader=yaml.FullLoader)
     return data
 
 def merge_configs(config, overrides):
@@ -425,7 +438,7 @@ def get_merged_config(create_if_needed=True, local_overrides_path=None, suppress
         config.mini_mode = False
     else:
         # get mini_mode value from default config (modified further below)
-        config.mini_mode = config.get("general", "advanced-mode")
+        config.mini_mode = not config.get("general", "advanced-mode")
         if config.mini_mode:
             suppress_warning = True
 
