@@ -213,14 +213,15 @@ class DGD(object):
         self.name_hparam_dict = {}  # This should go away.
         self.hparams = []
 
-        in_hp_section = False
+        # REVIEW: we no longer support sections in this call (all hparams are searcharble)
+        in_hp_section = True    #  False
         comment = None
 
         for record in records:
 
             name_string = record["name"]
             value_string = record["value"]
-            
+
             if "in_hp_section" in record:
                 # legacy text source 
                 in_hp_section = record["in_hp_section"]
@@ -270,6 +271,7 @@ class DGD(object):
                     continue
 
                 if hp_value not in hparam.value_setting_dict:
+                    # REVIEW: why are we landing here?
                     ignore_run = True
                     break  # Skip this run. Its value must have been removed from config.txt.
 
@@ -384,7 +386,7 @@ class DGD(object):
                 y1 = y2
         return x1
 
-    def assign_settings(selfm, hp_config):
+    def assign_settings(self, hp_config):
         console.print('assigning settings')
         return
 
