@@ -6,6 +6,7 @@
 import os
 import sys
 import json
+import time
 import importlib
 import numpy as np
 
@@ -22,6 +23,8 @@ class HParamSearch():
 
     def __init__(self, is_aml=False):
         self.is_aml = is_aml
+        self.run_history = []
+        self.end_id = 0
 
     # main ENTRY POINT 
     def process_child_hparams(self, child_name, store, context, parent):
@@ -79,12 +82,7 @@ class HParamSearch():
 
         # algorithm-specific processing
         search_type = context.search_type
-
-        if search_type == "bayesian":
-            # use original size=() for randint
-            records = self.parse_hp_config_yaml(hparams, hp_size=True)
-        else:
-            records = self.parse_hp_config_yaml(hparams, hp_size=False)
+        records = self.parse_hp_config_yaml(hparams, search_type)
 
         # get code_path for search_type from hpsearch_providers
         arg_dict = self.hp_search_core(context, search_type, store, run_name, records)
@@ -101,13 +99,22 @@ class HParamSearch():
 
         if need_runs:
             # this could be a lengthly delay
+            started = time.time()
             runs = self.get_completed_runs(store, context) 
+            elapsed = time.time() - started
+            console.print("hp_search_core: {:,} runs (new={:,}, elapsed: {:.2f} secs)".format(len(runs), self.new_run_count, elapsed), flush=True)
 
         arg_dict = impl.search(run_name, store, context, space_records, runs=runs)
-        console.print("\n---- search_type={}, returned arg_dict={} ----\n".format(search_type, arg_dict))
+
+        # if not testing, output some debug info
+        if search_type != "random":
+            console.print("\n---- search_type={}, returned arg_dict={} ----\n".format(search_type, arg_dict))
+
         return arg_dict
 
-    def parse_hp_config_yaml(self, hparams, hp_size=None):
+    def parse_hp_config_yaml(self, hparams, search_type):
+        # use original size=() for randint on hyperopt (bayesian) search
+        hp_size = (search_type == "bayesian")
 
         records = []
         for prop, value in hparams.items():
@@ -154,26 +161,35 @@ class HParamSearch():
 
         return insert_index
 
-    def get_completed_runs(self, store, context, metric_reports=False):
+    def get_completed_runs(self, store, context):
         if context.aggregate_dest == "job":
             filter_dict = {"job_id": context.dest_name}
         elif context.aggregate_dest == "experiment":
             filter_dict = {"exper_name": context.dest_name}
 
-        # for now, don't use cache (return all completed runs in job/experiment)
-        filter_dict["end_id"] = {"$gt": 0}
+        # we are using in-memory cache (self.run_history)
+        filter_dict["end_id"] = {"$gt": self.end_id}
 
         # fields needed for dgd and bayesian searches
-        fields_dict = {"run_name": 1, "hparams": 1, context.primary_metric: 1}
-
-        if metric_reports:
-            fields_dict["metrics"] = 1
+        metric = "metrics." + context.primary_metric
+        fields_dict = {"run_name": 1, "end_id": 1, "hparams": 1, metric: 1}
 
         fn_cache = "hparams_search.cache"
 
         # get all completed runs from MONGO 
-        runs = store.get_all_runs(context.aggregate_dest, context.ws, context.dest_name, filter_dict, fields_dict, use_cache=True, 
-            fn_cache=fn_cache)
+        new_runs = store.get_all_runs(context.aggregate_dest, context.ws, context.dest_name, filter_dict, fields_dict, use_cache=False)
+
+        # merge new runs with previous
+        runs = self.run_history + new_runs
+        
+        # update cache
+        self.run_history = runs
+        self.new_run_count = len(new_runs)
+
+        if self.new_run_count:
+            end_ids = [run["end_id"] for run in new_runs]
+            self.end_id = max(end_ids)
+
         return runs
    
     def apply_hparam_set(self, arg_dict, cmd_parts, context):
