@@ -5,32 +5,37 @@
 # impl_compute.py: implementation of XT compute commands
 import os
 import sys
+import json
 import time
 import datetime
 import numpy as np 
 
-from .client import Client
-from .runner import Runner
-from .console import console
+from xtlib.client import Client
+from xtlib.runner import Runner
+from xtlib.console import console
 from xtlib.storage.store import Store
-from .cmd_core import CmdCore
-from .impl_base import ImplBase
-from xtlib.attach import Attach
-from .backends import backend_aml 
+from xtlib.cmd_core import CmdCore
+from xtlib.impl_base import ImplBase
+from xtlib.backends import backend_aml 
 from xtlib.xt_client import XTClient
-from .helpers.scanner import Scanner
-from .qfe import inner_dispatch, get_dispatch_cmd, Dispatcher
-from .qfe import command, argument, option, flag, root, clone
-from .qfe import hidden, example, command_help, keyword_arg
+from xtlib.helpers.scanner import Scanner
+from xtlib.qfe import inner_dispatch, get_dispatch_cmd, Dispatcher
+from xtlib.qfe import command, argument, option, flag, root, clone
+from xtlib.qfe import hidden, example, command_help, keyword_arg
+from xtlib.helpers.key_press_checker import KeyPressChecker, single_char_input
 
+from xtlib import qfe
 from xtlib import utils
 from xtlib import errors
 from xtlib import capture
 from xtlib import pc_utils
+from xtlib import constants
 from xtlib import file_utils
 from xtlib import job_helper
 from xtlib import run_helper
+from xtlib import process_utils
 from xtlib import box_information
+
 '''
 This module implements the following commands:
 
@@ -145,38 +150,169 @@ class ImplCompute(ImplBase):
             backend.view_status(run_name, workspace, job, monitor, escape_secs, auto_start, 
                 stage_flags, status, max_finished)
 
-    
-    #---- ATTACH command ----
-    @argument(name="run-name", help="The name of the run to attach")
-    @option("workspace", default="$general.workspace", type=str, help="the workspace that the run resides within")
-    @option("escape", type=int, help="the number of seconds to remain attached")
-    @example(task="attach to run68 in the current workspace", text="xt attach run68")
-    @command(help="attaches the streaming output of the specified run to the console")
-    def attach(self, workspace, run_name, escape):
-        run_name, actual_ws = run_helper.parse_run_name(workspace, run_name)
-        is_aml = self.is_aml_ws(actual_ws)
-
-        if is_aml:
-            self.azure_ml.attach_to_run(actual_ws, run_name)
-        else:
-            #self.client.monitor_attach_run(actual_ws, run_name, escape=escape)
-            cs, box_secret = run_helper.get_client_cs(self.core, actual_ws, run_name)
-            if not cs:
-                console.print("could not find info for run: {}/{}".format(actual_ws, run_name))
-            else:
-                with XTClient(self.config, cs, box_secret) as xtc:
-                    attach = Attach(xtc)
-                    attach.monitor_attach_run(actual_ws, run_name, escape=escape)
-
-                    # xtc.connect()
-                    # attach.simple_attach(actual_ws, run_name)
-
     #---- MONITOR command ----
-    @argument(name="run-name", help="The name of the run to attach")
+    # other candidate names for this command: watch, attach
+    @argument(name="name", help="The name of the run or job to be monitored")
+    @option("escape", type=int, default=0, help="breaks out of attach or --monitor loop after specified # of seconds")
+    @flag("jupyter", help="to monitor a job from a jupyter notebook (AML only)")
+    @option("log-name", default=None, help="the name of the log file to be monitored")
+    @option("node-index", default=0, type=int, help="the node index for multi-node jobs")
+    @option("sleep", default=1, type=float, help="the number of seconds between download calls")
     @option("workspace", default="$general.workspace", type=str, help="the workspace that the run resides within")
-    @example(task="monitor the AML run run544 from exper5", text="xt monitor exper5.run544")
-    @command(help="creates a Python notebook to monitor the specified Azure ML run")
-    def monitor(self, workspace, run_name):
+    @example(task="monitor job3321's primary log file", text="xt monitor job3321")
+    @command(help="view a job's log file, as it grows in real-time")
+    def monitor(self, name, escape=None, jupyter=None, log_name=None, node_index=None, sleep=1, workspace=None):
+        if jupyter:
+            return self.monitor_with_jupyter(workspace, name)
+
+        if name.startswith("job"):
+            return self.monitor_job_node(name, jupyter, sleep, node_index, log_name, escape) 
+
+        if name.startswith("run"):
+            rr = run_helper.get_run_record(self.store, workspace, name)
+            job_id = rr["job_id"]
+            node_index = rr["node_index"]
+            return self.monitor_job_node(job_id, jupyter, sleep, node_index, log_name, escape) 
+
+        errors.syntax_error("name must be a job or run name: {}".format(name))
+
+    def monitor_job_node(self, job_id, jupyter, sleep, node_index, log_name, escape):
+
+        if node_index is None:
+            node_index = 0
+
+        backend, job_info = job_helper.get_job_backend(self.store, self.core, job_id)
+
+        node_id = utils.node_id(node_index)
+        service_info_by_node = job_info["service_info_by_node"]  
+        service_node_info = service_info_by_node[node_id]
+
+        service_name = backend.get_name()
+        node_count = len(job_info["service_info_by_node"])
+
+        console.print("==> monitoring: {}, node{} [{}] (press escape or control-c to exit, +/- to change node)" \
+            .format(job_id, node_index, service_name), flush=True)
+
+        # the monitoring loop
+        start_offset = 0
+        simple_status = None
+        service_status = None
+        kb_sleep = .1       # be quick to respond to user's key presses
+        sleeps_per_call = max(1, sleep//kb_sleep)
+        sleep_count = 0
+        ch = None
+        first_call = True
+        offset_by_node = {}
+        node_id = utils.node_id(node_index)
+        first_text_of_stream = True
+        display_count = 0
+        started = time.time()
+
+        try:
+            with KeyPressChecker() as checker:
+
+                while simple_status != "completed":
+                    ch = checker.getch_nowait()
+                    #print(ch, end=" ")
+
+                    if ch == constants.ESCAPE or ch == constants.CONTROL_C:
+                        break
+
+                    if ch in ["+", "-"] and node_count > 1:
+                        # save current context
+                        offset_by_node[node_id] = start_offset
+
+                        # increment node_index
+                        delta = 1 if ch == "+" else -1
+                        node_index = (delta + node_index) % node_count
+                        node_id = utils.node_id(node_index)
+
+                        # set new context
+                        service_node_info = service_info_by_node[node_id]
+                        start_offset = offset_by_node[node_id] if node_id in offset_by_node else 0
+                        first_text_of_stream = not start_offset
+                        console.print("==> switching to: node{}".format(node_index))
+
+                    if ch == "q":
+                        # diagnostic aid (undocumented, only for pool service)
+                        console.print("{} service queue:".format(node_id), flush=True)
+
+                        entries = backend.get_service_queue_entries(service_node_info)
+                        if entries is None:
+                            console.print("  <not supported for this service")
+                        else:
+                            if entries:
+                                for entry in entries:
+                                    marker = "*" if entry["current"] else " "
+                                    console.print("  {} {}".format(marker, entry["name"]))
+                            else:
+                                console.print("  <no entries>")
+
+                    if sleep_count == sleeps_per_call:
+
+                        # time to read the log file
+                        result = backend.read_log_file(service_node_info, log_name=log_name, start_offset=start_offset)
+
+                        new_text = result["new_text"]
+                        new_simple = result["simple_status"]
+                        new_service = result["service_status"]
+                        start_offset = result["next_offset"]
+                        new_log = result["log_name"]
+
+                        if new_log != log_name:
+                            log_name = new_log                
+                            console.print("==> node{} streaming log: {}".format(node_index, log_name))
+                            first_text_of_stream = True
+
+                        if new_service != service_status:
+                            service_status = new_service
+                            simple_status = new_simple
+                            console.print("==> node{} status: {} ({})".format(node_index, service_status, simple_status))
+
+                        if new_text:
+                            if node_count > 1:
+                                # prepend each new line with node_id
+                                prefix = node_id + ": "
+                                new_text = new_text.replace("\n", "\n" + prefix)
+
+                                if first_text_of_stream:
+                                    new_text = prefix + new_text
+                                    first_text_of_stream = False
+
+                            console.print(new_text, end="")
+                            display_count += 1
+
+                        first_call = False
+                        sleep_count = 0
+
+                    time.sleep(kb_sleep)
+                    sleep_count += 1
+
+                    if escape:
+                        # have we exceeded max time in monitoring?
+                        elapsed = time.time() - started
+                        if elapsed >= escape:
+                            break
+
+        except KeyboardInterrupt:
+           ch = constants.CONTROL_C
+
+        show_final_status = (display_count > 0)
+
+        if ch == constants.ESCAPE:
+            console.print("==> monitoring cancelled (escape key detected)")
+
+        elif ch == constants.CONTROL_C:
+            console.print("==> monitoring cancelled (control-c detected)")
+            ch = single_char_input("do you want to cancel the job? (y/n): ")
+            if ch == "y":
+                self.cancel_job(job_id)
+                show_final_status = False
+
+        if show_final_status:
+            console.print("==> node{} status: {} ({})".format(node_index, service_status, simple_status))
+
+    def monitor_with_jupyter(self, workspace, run_name):
         if not self.is_aml_ws(workspace):
             errors.combo_error("the monitor command is only supported for Azure ML runs")
 
@@ -303,18 +439,18 @@ class ImplCompute(ImplBase):
             # change back to original dir
             os.chdir(prev_cwd)
 
-    #---- STOP CONTROLLER command ----
-    @option("box", default="local", type=str, help="the name of the box")
-    @option("pool", type=str, help="the name of the pool of boxes")
-    @example(task="stop the controller process on the local machine", text="xt stop controller")
-    @example(task="terminate the controller process on box 'vm10'", text="xt stop controller --box=vm10")
-    @command(help="stops the XT controller process on the specified box")
-    def stop_controller(self, box, pool):
+    # #---- STOP CONTROLLER command ----
+    # @option("box", default="local", type=str, help="the name of the box")
+    # @option("pool", type=str, help="the name of the pool of boxes")
+    # @example(task="stop the controller process on the local machine", text="xt stop controller")
+    # @example(task="terminate the controller process on box 'vm10'", text="xt stop controller --box=vm10")
+    # @command(help="stops the XT controller process on the specified box")
+    # def stop_controller(self, box, pool):
 
-        boxes, pool_info, service_type = box_information.get_box_list(self.core, box=box, pool=pool)
-        #console.print("boxes=", boxes, ", is_azure_pool=", is_azure_pool)
+    #     boxes, pool_info, service_type = box_information.get_box_list(self.core, box=box, pool=pool)
+    #     #console.print("boxes=", boxes, ", is_azure_pool=", is_azure_pool)
 
-        self.client.cancel_controller_by_boxes(boxes)
+    #     self.client.cancel_controller_by_boxes(boxes)
 
     def print_cancel_results(self, cancel_results_by_boxes, run_summary=None, is_aml=False):
         #console.print("cancel_results_by_boxes=", cancel_results_by_boxes)
@@ -411,40 +547,27 @@ class ImplCompute(ImplBase):
         self.print_cancel_results(cancel_results_by_box)
 
     #---- CANCEL JOB command ----
-    @argument("job", help="the name of the job to cancel")
+    @argument("job-id", help="the name of the job to cancel")
     @example(task="cancel all runs in job3125", text="xt cancel job job3125")
     @command(kwgroup="cancel", kwhelp="cancels active runs, specified by name, job, or target", help="cancels the specified job and its active or queued runs")
-    def cancel_job(self, job):
+    def cancel_job(self, job_id):
 
-        mongo = self.store.get_mongo()
+        console.print("cancelling {}:".format(job_id))
 
-        filter_dict = {}
-        filter_dict["job_id"] = {"$in": [job]}
-        fields_dict = None    # get entire document  #{"compute": 1}
+        service_job_info, service_info_by_node, backend = job_helper.get_service_job_info(self.store, self.core, job_id)
+        result_by_node = backend.cancel_job(service_job_info, service_info_by_node)
 
-        cursor = mongo.get_info_for_jobs(filter_dict, fields_dict)
-        records = list(cursor)
-        if len(records) != 1:
-            errors.store_error("unknown job_id={}".format(job))
+        for node_id, result in result_by_node.items():
+            console.print("  {}: {}".format(node_id, result))
 
-        first = records[0]
-        compute = first["compute"]
-        runs_by_box = first["runs_by_box"]
+        # wrap up runs associated with each node
+        wrapup_count = 0
 
-        backend = self.core.create_backend(compute)
-        compute_info = self.config.get_compute_def(compute)
-        service_name = compute_info["service"]
+        for node_id, result in result_by_node.items():
+            runs = run_helper.wrapup_runs_by_node(self.store, job_id, node_id, result)
+            wrapup_count += len(runs)
 
-        cancel_results_by_box = backend.cancel_runs_by_job(job, runs_by_box)
-
-        # if service_name == "pool":
-        #     # feed backend 1 box at a time
-        #     for box_name in runs_by_box:
-        #         cancel_results = backend.cancel_runs_by_job(job, box_name)
-        #         cancel_results_by_group[box_name] = cancel_results
-        # else:
-
-        self.print_cancel_results(cancel_results_by_box)
+        console.print("  runs wrapped-up: {}".format(wrapup_count))
 
     #---- CANCEL ALL command ----
     @argument("target", help="the name of the compute target whose runs will be cancelled")
@@ -519,7 +642,7 @@ class ImplCompute(ImplBase):
                 if xtc.connect():
                     elapsed = xtc.get_controller_elapsed()
                     xt_version = xtc.get_controller_xt_version()
-                    max_runs = xtc.get_controller_max_runs()
+                    max_runs = xtc.get_controller_concurrent()
                     ip_addr = xtc.get_controller_ip_addr()
                     status_text = xtc.get_runs(stage_flags)
 
@@ -580,8 +703,7 @@ class ImplCompute(ImplBase):
     @hidden("after-omit", default="$after-files.after-omit", help="the files and directories to omit from after uploading")
     @flag("after-upload", default="$after-files.after-upload", help="when true, the after files are upload when the run completes")
     @hidden("option-prefix", default="$hyperparameter-search.option-prefix", help="the prefix to be used for specifying hyperparameter options to the script")
-    @flag("attach", default="$general.attach", help="when True, the XT controller is also used on Azure ML runs")
-    @option("cluster", type=str, help="the name of the Philly cluster to be used")
+    @option("cluster", help="the name of the Philly cluster to be used")
     @hidden("code-dirs", default="$code.code-dirs", help="paths to the main code directory and dependent directories")
     @hidden("code-omit", default="$code.code-omit", help="the list wildcard patterns to omit uploading from the code files")
     @flag("code-upload", default="$code.code-upload", help="when true, code is uploaded to job and download for each run of job")
@@ -598,11 +720,11 @@ class ImplCompute(ImplBase):
     @flag("direct-run", default="$general.direct-run", help="when True, the script is run without the controller (on Philly or AML)")
     @flag("distributed", default="$general.distributed", help="when True, the multiple nodes will be put into distributed training mode")
     @hidden("distributed-training", default="$aml-options.distributed-training", help="the name of the backend process to use for distributed training")
+    @option("docker", help="the docker entry to use with the target (from one of the entries in the config file [dockers] section")
     @flag("dry-run", help="when True, the planned runs are displayed but not submitted")
     @hidden("early-policy", default="$early-stopping.early-policy", help="the name of the early-stopping policy to use (bandit, median, truncation, none)")
     @option("escape", type=int, default=0, help="breaks out of attach or --monitor loop after specified # of seconds")
     @hidden("env-vars", default="$general.env-vars", help="the environment variable to be passed to the run when it is launched")
-    @option("docker", help="the docker entry to use with the target (from one of the entries in the config file [dockers] section")
     @hidden("evaluation-interval", default="$early-stopping.evaluation-interval", type=int, help="the frequencency (# of metric logs) for testing the policy")
     @option("experiment", default="$general.experiment", type=str, help="the name of the experiment to create this run under")
     @flag("fake-submit", help="when True, we skip creation of job and runs and the submit (used for internal testing)")
@@ -621,9 +743,11 @@ class ImplCompute(ImplBase):
     @hidden("mirror-dest", default="$logging.mirror-dest", help="the location where mirrored information is store (none, storage, grok)")
     @hidden("mirror-files", default="$logging.mirror-files", help="the wildcard path that specifies files to be mirrored")
     @option("model-action", default="$model.model-action", help="the model action to take on the target, before run is started")
+    @hidden("model-local", default="$model.model-local", help="the path on the local machine specifying where the model for this job resides")
     @hidden("model-share-path", default="$model.model-share-path", help="the model share name for the model")
     @flag("model-writable", default="$model.model-writable", help="when true, a mounted model path can be written to")
-    @flag("monitor", help="when True, a Jupyter notebook is created to monitor the run")
+    @flag("jupyter-monitor", help="when True, a Jupyter notebook is created to monitor the run")
+    @option("monitor", default="$general.monitor", values=["new", "same", "none"], help="how to monitor primary run of the new job")
     @flag("multi-commands"   , help="the script file contains multiple run commands (one per line)")
     @option("nodes", type=int, help="the number of normal (non-preemptable) nodes to allocte for this run")
     @option("parent-script", type=str, help="path of script used to initialize the target for repeated runs of primary script")
@@ -638,16 +762,15 @@ class ImplCompute(ImplBase):
     @option("schedule", default="static", values=["static", "dynamic"], help="specifies if runs are pre-assigned to each node or allocate on demand")
     @option("search-type", values=["random", "grid", "bayesian", "dgd"], default="$hyperparameter-search.search-type", help="the type of hyperparameter search to perform")
     @option("seed", default=None, help="the random number seed that can be used for reproducible HP searches")
-    @flag("show-controller", default="$internal.show-controller", help="displays output of local or pool boxes in an locla command window")
     @option("sku", help="the name of the Philly SKU to be used (e.g, 'G1')")
     @hidden("slack-factor", default="$early-stopping.slack-factor", type=float, help="(bandit only) specified as a ratio, the delta between this eval and the best performing eval")
     @hidden("slack-amount", default="$early-stopping.slack-amount", type=float, help="(bandit only) specified as an amount, the delta between this eval and the best performing eval")
     @hidden("storage", default="$xt-services.storage", help="name of storage service to be used for this run")
     @option("submit-logs", default=None, help="specifies a directory to which log files for the submit are saved")
     @option("target", default="$xt-services.target", help="one of the user-defined compute targets on which to run")
-    @hidden("truncation-percentage", default="$early-stopping.truncation-percentage", type=float, help="truncation only) percent of runs to cancel at each eval interval")
+    @hidden("truncation-percentage", default="$early-stopping.truncation-percentage", type=float, help="(truncation only) percent of runs to cancel at each eval interval")
     @option("use-gpu", type=bool, default="$aml-options.use-gpu", help="when True, the gpu(s) on the nodes will be used by the run")
-    @hidden("username", default="$general.username", help="the username to log as the author of this run")
+    @option("username", default="$general.username", help="the username to log as the author of this run")
     @hidden("user-managed", type=bool, default="$aml-options.user-managed", help="if true, it implies that the local machine or VM will be managed by the user")
     @option("vc", help="the name of the Philly virtual cluster to be used")
     @option("vm-size", help="the type of Azure VM computer to run on")
@@ -656,8 +779,8 @@ class ImplCompute(ImplBase):
     @flag("xtlib-upload", default="$code.xtlib-upload", help="when True, local source code for xtlib is included in the source code snapshot ")
     
     @example(task="run the script miniMnist.py", text="xt run miniMnist.py")
-    @example(task="run the linux command 'sleep3d' on philly", text="xt --target=philly --code-upload=0 run sleep 3d")
-    @command(options_before_cmd=True, keyword_optional=False, pass_by_args=True, help="submits a script or executable file for running on the specified compute target")
+    @example(task="run the linux command 'sleep3d' on philly", text="xt run --target=philly --code-upload=0 sleep 3d")
+    @command(options_before_args=True, keyword_optional=False, pass_by_args=True, help="submits a script or executable file for running on the specified compute target")
     def run(self, args):
         '''
         The run command is used to run a program, python script, batch file, or shell script on 
@@ -676,21 +799,18 @@ class ImplCompute(ImplBase):
         args["xt_cmd"] = "xt " + cmd if cmd else ""
 
         runner = Runner(self.config, self.core)
-        runner.process_run_command(args)
 
-    # TEMP (dup of runner.py function)
-    def get_script_dir(self, cmd_parts):
-        script_dir = None
+        cmds, run_specs, using_hp, using_aml_hparam, sweeps_text, pool_info, job_id = \
+            runner.process_run_command(args)
 
-        parts = cmd_parts
-        for i, part in enumerate(parts):
-            path = os.path.realpath(part)
-            if os.path.isfile(path):
-                script_dir = os.path.dirname(path)
-                parts[i] = os.path.basename(path)
-                break
+        monitor = args["monitor"]
+        escape = args["escape"]
 
-        return script_dir
-    
+        if monitor == "same":
+            workspace = args["workspace"]
+            self.monitor(name=job_id, workspace=workspace, escape=escape)
 
+        elif monitor == "new":
+            cmd = "xt --echo monitor {}".format(job_id)
+            process_utils.run_cmd_in_new_console(cmd)
 
