@@ -378,7 +378,7 @@ class ImplHelp(ImplBase):
         examples = cmd_info["examples"]
         see_alsos = cmd_info["see_alsos"]
         faqs = cmd_info["faqs"]
-        opts_before_cmd = cmd_info["options_before_cmd"]
+        options_before_args = cmd_info["options_before_args"]
 
         words = name.replace("_", " ")
         if cmd_info["keyword_optional"]:
@@ -394,8 +394,8 @@ class ImplHelp(ImplBase):
         if not syntax_only:
             console.print()
 
-        if opts_before_cmd:
-            usage = "Usage: {}".format(self.name) + opts_text + words + args_text
+        if options_before_args:
+            usage = "Usage: {}".format(self.name) + words + opts_text + args_text
         else:
             usage = "Usage: {}".format(self.name) + words + args_text + opts_text
 
@@ -421,7 +421,7 @@ class ImplHelp(ImplBase):
         else:
             # show each option on its own line with a short description
             text = ""
-            if opts_before_cmd:
+            if options_before_args:
                 text += self.gen_options(options)
                 text += self.gen_args(args)
             else:
@@ -502,7 +502,7 @@ class ImplHelp(ImplBase):
         examples = cmd_info["examples"]
         see_alsos = cmd_info["see_alsos"]
         words = name.replace("_", " ")
-        opts_before_cmd = cmd_info["options_before_cmd"]
+        options_before_args = cmd_info["options_before_args"]
 
         if cmd_info["keyword_optional"]:
             words = "[ " + words + " ]"
@@ -534,8 +534,8 @@ class ImplHelp(ImplBase):
         # put it all TOGETHER 
         usage = "Usage::\n\n    " 
 
-        if opts_before_cmd:
-            usage += opts_text + cmd_words + args_text
+        if options_before_args:
+            usage += cmd_words + opts_text + args_text
         else:
             usage += cmd_words + args_text + opts_text
 
@@ -550,7 +550,7 @@ class ImplHelp(ImplBase):
 
         text += help_text + "\n"
         
-        if opts_before_cmd:
+        if options_before_args:
             text += self.gen_options(options, True)
             text += self.gen_args(args, True)
         else:
