@@ -267,35 +267,30 @@ class ImplStorage(ImplBase):
                 console.print("  " + str(name))
 
     #---- VIEW CONSOLE command ----
-    @argument(name="target", help="the name of the run or job")
+    @argument(name="name", help="the name of the run or job")
     @option(name="workspace", default="$general.workspace", help="the workspace that the run resides in")
     @option(name="node-index", default=0, help="the node index for the specified job")
     @example(task="view the console output for run26 in the curious workspace", text="xt view console curions/run26")
     @example(task="view the console output for job201, node3", text="xt view console job201 --node-index=3")
     @command(kwgroup="view",  help="view console output for specified run")
-    def view_console(self, target, workspace, node_index):
-        if target.startswith("job"):
-            # ji_text = self.store.read_job_info_file(target)
-            # job_info = json.loads(ji_text)
-            # if not utils.is_philly_job(job_info):
-            #     errors.combo_error("view console currently only supports Philly jobs")
-            # result, philly_job_id = self.philly.get_philly_job_info(target, job_info, "log", False)
-            # console.print("console output for: {} (philly {})".format(target, philly_job_id))
-            # console.print(result)
-            workspace = job_helper.validate_job_name_with_ws(self.store, target, True)
+    def view_console(self, name, workspace, node_index):
+        if name.startswith("job"):
+            # treat target as job_name
+            job_name = name
+            workspace = job_helper.validate_job_name_with_ws(self.store, job_name, True)
             fn = "node-{}/after/stdout.txt".format(node_index)
 
-            if not self.store.does_job_file_exist(target, fn):
-                console.print("job '{}' has no file'{}'".format(target, fn))
+            if not self.store.does_job_file_exist(job_name, fn):
+                console.print("job '{}' has no file'{}'".format(job_name, fn))
             else:
-                console.print("{} for {}:\n".format(fn, target))
+                console.print("{} for {}:\n".format(fn, job_name))
 
-                text = self.store.read_job_file(target, fn)
+                text = self.store.read_job_file(job_name, fn)
                 text = pc_utils.make_text_display_safe(text)
                 console.print(text)            
         else:
             # treat target as run_name
-            run_name = target
+            run_name = name
             is_aml = False # self.is_aml_ws(workspace)
             ws, run_name, full_run_name = run_helper.validate_run_name(self.store, workspace, run_name, parse_only=is_aml)
 
@@ -358,33 +353,70 @@ class ImplStorage(ImplBase):
                 console.print(record)
 
     #---- VIEW METRICS command ----
-    @argument(name="run-name", help="the name of the run")
+    @argument(name="runs", type="str_list", help="a comma separated list of runs, jobs, or experiments", required=True)
+    @argument(name="metrics", type="str_list", required=False, help="optional list of metric names")
     @option(name="workspace", default="$general.workspace", help="the workspace that the run resides in")
+    @option(name="steps", type="int_list", help="show metrics only for the specified steps")
     @option(name="export", type="str", help="will create a tab-separated file for the report contents")
+    @flag(name="merge", help="will merge all datasets into a single table")
+    @option(name="hparams", type="str_list", help="will list the specified hyperparmeter names and values before the metrics")
     @example(task="view the logged metrics for run153 in the current workspace", text="xt view metrics run153")
     @command(kwgroup="view", help="view the set of logged metrics for specified run")
-    def view_metrics(self, run_name, workspace, export):
-        is_aml = self.is_aml_ws(workspace)
-        ws, run_name, full_run_name = run_helper.validate_run_name(self.store, workspace, run_name, parse_only=is_aml)
+    def view_metrics(self, runs, workspace, steps, merge, metrics, hparams, export):
 
-        records = self.store.get_run_log(ws, run_name)
-        console.print("metrics for {}:\n".format(full_run_name))
+        args = {"run_list": runs, "workspace": workspace, "all": True, "sort_col": "name", 
+            "max_runs": None, "columns": ["run", "hparams.*", "metrics.*"]}
 
-        metric_sets = run_helper.build_metrics_sets(records)
-        for i, ms in enumerate(metric_sets):
-            lb = ReportBuilder(self.config, self.store, client=None)
+        # actual store col names (vs. user-level names) used here
+        col_dict = {"run_name": 1, "node_index": 1, "job_id": 1, "exper_name": 1, "ws": 1, "hparams": 1, "log_records": 1}
 
-            if export:
-                sep_char = "\t"
-                count = lb.export_records(export, ms["records"], ms["keys"], ms["keys"], sep_char)
-                console.print("report exported to: {} ({} rows)".format(export, count-1))
-            else:
-                console.print("Dataframe {}:".format(1+i))
+        run_log_records, using_default_last, user_to_actual, available, builder, last, std_cols_desc = \
+            run_helper.get_filtered_sorted_limit_runs(self.store, self.config, False, col_dict=col_dict, args=args)
 
-                text, row_count = lb.build_formatted_table(ms["records"], ms["keys"])
-                # indent text 2 spaces on each line
-                text = "  " + text.replace("\n", "\n  ")
-                console.print(text)
+        for rr in run_log_records:
+            console.print("\n{}:".format(rr["_id"]), end="")
+
+            if hparams:
+                #console.print("  hyperparameters:")
+                rrh = rr["hparams"]
+                first_value = True
+
+                for name in hparams:
+                    value = utils.safe_value(rrh, name)
+                    if first_value:
+                        console.print(" (", end="")
+                        first_value = False
+                    else:
+                        console.print(", ", end="")
+
+                    console.print("{}: {}".format(name, value), end="")
+
+                # finish line and skip a line
+                if first_value:
+                    console.print("\n")
+                else:
+                    console.print(")\n")
+ 
+            # build the metric sets
+            log_records = rr["log_records"]
+            metric_sets = run_helper.build_metrics_sets(log_records, steps, merge, metrics)
+            just_one = len(metric_sets) == 1
+
+            for i, ms in enumerate(metric_sets):
+                lb = ReportBuilder(self.config, self.store, client=None)
+
+                if export:
+                    sep_char = "\t"
+                    count = lb.export_records(export, ms["records"], ms["keys"], ms["keys"], sep_char)
+                    console.print("report exported to: {} ({} rows)".format(export, count-1))
+                else:
+                    if not just_one:
+                        console.print("Dataframe {}:".format(1+i))
+
+                    text, row_count = lb.build_formatted_table(ms["records"], ms["keys"])
+                    # indent text 2 spaces on each line
+                    text = "  " + text.replace("\n", "\n  ")
+                    console.print(text)
 
     #---- PLOT command ----
     # args, flags, options
@@ -475,7 +507,7 @@ class ImplStorage(ImplBase):
     @option(name="workspace", default="$general.workspace", help="the workspace that the runs resides in")
     @option("response", default=None, help="the response to be used to confirm the directory deletion")
     @example(task="extract files from curious/run26 to c:\\run26_files", text="xt extract curious/run26 c:\\run26_files")
-    @command(help="copy all files associated with the run to the specified directory")
+    @command(help="download all files associated with the run to the specified directory")
     def extract(self, run_name, dest_dir, workspace, response, browse):
         is_aml = self.is_aml_ws(workspace)
         ws, run_name, full_run_name = run_helper.validate_run_name(self.store, workspace, run_name, parse_only=is_aml)
@@ -579,6 +611,8 @@ class ImplStorage(ImplBase):
     def view_blob(self, path, share, workspace, job, experiment, run):
 
         use_blobs = True
+        fs = self.create_file_accessor(use_blobs=True, share=share, ws_name=workspace, 
+            exper_name=experiment, job_name=job, run_name=run)
 
         text = fs.read_file(path)
         # if job:
@@ -693,6 +727,7 @@ class ImplStorage(ImplBase):
     @option(name="status", type="str_list", default="$run-reports.status", 
         values= ["created", "allocating", "queued", "spawning", "running", "completed", "error", "cancelled", "aborted", "unknown"], 
         help="match runs whose status is one of the values in the list")
+    @option(name="username", type="str", help="only show runs for the specified username")
     
     # report flags
     @flag(name="flat", help="do not group runs")
@@ -756,6 +791,7 @@ class ImplStorage(ImplBase):
     @option(name="precision", type=int, default="$run-reports.precision", help="set the number of factional digits for float values")
     @option(name="columns", type="str_list", default="$job-reports.columns", help="specify list of columns to include")
     @option(name="export", type="str", help="will create a tab-separated file for the report contents")
+    @option(name="username", type="str", help="only show runs for the specified username")
     
     # report flags
     @flag(name="reverse", help="reverse the sorted items")
@@ -979,7 +1015,7 @@ class ImplStorage(ImplBase):
 
             if len(blob_names) == 0:
                 console.print("no matching {} found in: {}".format(what, uri))
-                return
+                return 0
             elif len(blob_names) == 1:
                 what = "blob" if use_blobs else "file"
 
@@ -1196,9 +1232,9 @@ class ImplStorage(ImplBase):
 
 
     #---- CREATE TEAM command ----
-    @example(task="create a template for a new XT team", text="xt create team")
+    @example(task="create a template for a new XT team", text="xt create services template")
     @command(kwgroup="create", help="generate an Azure template for creating a set of resources for an XT Team")
-    def create_team(self):
+    def create_services_template(self):
         '''Once you have run this command to generate a team template, follow these instructions to complete the process:
         
         1. browse to the Azure Portal Custom Template Deployment page: https://ms.portal.azure.com/#create/Microsoft.Template
@@ -1340,7 +1376,7 @@ class ImplStorage(ImplBase):
     @keyword_arg(name="name", keywords=["xt", "controller", "quick-test"], help="name of the event log")
     @option(name="last", default=25, help="the number of most recent entries to display")
     @flag(name="all", help="specify to display all entries")
-    @example(task="view the most recent events in the XT log", text="xt view xt log")
+    @example(task="view the most recent events in the XT log", text="xt view events xt")
     @command(kwgroup="view", help="view formatted information in the XT event log")
     def view_events(self, name, last, all):
         
