@@ -3,6 +3,8 @@
 # Licensed under the MIT license.
 #
 # job_helper.py: functions needed for processing job-related information
+import json
+import time
 import fnmatch
 
 from .console import console
@@ -78,7 +80,7 @@ def build_filter_part(fd, args, arg_name, store_name):
 
 def build_job_filter_dict(job_list, user_to_actual, builder, workspace, args):
     fd = {}
-    option_filters = ["experiment", "target", "service_type"]   # "application"
+    option_filters = ["experiment", "target", "service_type", "username"]   # "application"
 
     if job_list:
         # filter by specified job names
@@ -338,3 +340,51 @@ def get_client_cs(core, job_id, node_index):
     cs_plus = {"cs": cs, "box_secret": box_secret, "job": job}
     return cs_plus
 
+def get_job_records(store, job_names, fields_dict=None):
+    ''' return job records for specified job names'''
+
+    mongo = store.get_mongo()
+
+    filter_dict = {}
+    filter_dict["job_id"] = {"$in": job_names}
+
+    job_records = mongo.get_info_for_jobs(filter_dict, fields_dict)
+
+    return job_records
+    
+def get_job_record(store, job_id, fields_dict = None):
+    job_records = get_job_records(store, [job_id], fields_dict)
+    if not job_records:
+        errors.store_error("job {} does not exist".format(job_id))
+    jr = job_records[0]
+    return jr
+    
+def get_job_backend(store, core, job_id):
+    
+    # FYI: getting job record from storage: 2.75 secs, from mongo: 2.39 secs (mongo slightly faster)
+    job_info = get_job_record(store, job_id, {"pool_info": 1, "service_info_by_node": 1})
+
+    target = job_info["pool_info"]["name"]
+    backend = core.create_backend(target)
+
+    return backend, job_info
+
+def get_service_node_info(job_info, node_index):
+    
+    node_id = utils.node_id(node_index)
+    service_info_by_node = job_info["service_info_by_node"]  
+    service_node_info = service_info_by_node[node_id]
+
+    return service_node_info
+
+def get_service_job_info(store, core, job_id):
+
+    job_info = get_job_record(store, job_id, {"pool_info": 1, "service_info_by_node": 1, "service_job_info": 1})
+
+    target = job_info["pool_info"]["name"]
+    backend = core.create_backend(target)
+
+    service_info_by_node = job_info["service_info_by_node"]  
+    service_job_info = job_info["service_job_info"]  
+
+    return service_job_info, service_info_by_node, backend
