@@ -311,19 +311,27 @@ class PlotBuilder():
         y_values = None
         stat_values = None
 
+        '''
+        Note: a specific y_col could exist in different data_frames, depending
+        on the other columns logged with in during each run.  So, don't stop
+        searching on the first match with y_col - keep going until we get a 
+        matching set of group_name records also.
+        '''
         for cols, df in data_frames_by_cols.items():
             if y_col in df.columns:
                 # filter values for specified run name
                 df = df[ df[self.group_by]==group_name ]
+                record_count = len(df.index)
 
-                y_values = df[ y_col ].to_numpy(dtype=float)
+                if record_count:
+                    y_values = df[ y_col ].to_numpy(dtype=float)
 
-                if x_col and x_col in df.columns:
-                    x_values = df[ x_col ].to_numpy(dtype=float)
+                    if x_col and x_col in df.columns:
+                        x_values = df[ x_col ].to_numpy(dtype=float)
 
-                if stat_col and stat_col in df.columns:
-                    stat_values = df[ stat_col ].to_numpy(dtype=float)
-                break
+                    if stat_col and stat_col in df.columns:
+                        stat_values = df[ stat_col ].to_numpy(dtype=float)
+                    break
 
         return x_values, y_values, stat_values
 
@@ -345,12 +353,18 @@ class PlotBuilder():
             sns.set_style(self.style)
 
         # decide how layout, titles, etc. will be set
-        for dfx in data_frames_by_cols.values():
-            group_names = dfx[self.group_by].unique() 
+        group_names = set()
 
-            # first dataframe is sufficient
-            break
+        # gather group names (usually all are in the first dataset, but not always)
+        for dfx in data_frames_by_cols.values():
+            name_list = dfx[self.group_by].unique()
+            group_names.update(set(name_list))
         
+        group_names = list(group_names)
+
+        # this will sort the group names in a number-smart way 
+        group_names.sort(key=utils.natural_keys)
+
         group_count = len(group_names)
         col_count = len(self.col_names)
 
@@ -658,7 +672,7 @@ class PlotBuilder():
 
         for ms in metric_sets:
             keys = ms["keys"]
-            omits = [x_col, constants.STEP_NAME]
+            omits = [x_col, constants.STEP_NAME, constants.TIME]
 
             for key in keys:
                 if not key in omits:
