@@ -178,7 +178,7 @@ def root(help=""):
     return decorator_root
 
 # COMMAND decorator processor
-def command(name=None, group=None, kwgroup=None, kwhelp=None, options_before_cmd=False, keyword_optional=False, pass_by_args=False, help=""):
+def command(name=None, group=None, kwgroup=None, kwhelp=None, options_before_args=False, keyword_optional=False, pass_by_args=False, help=""):
     '''
     builds a nested dictionary of name parts for multi-word commands 
     and their associated functions.
@@ -210,7 +210,7 @@ def command(name=None, group=None, kwgroup=None, kwhelp=None, options_before_cmd
                 dd[name_part] = {}
             dd = dd[name_part]
 
-        cmd_info =  {"name": cmd_name, "options_before_cmd": options_before_cmd, "keyword_optional": keyword_optional, "pass_by_args": pass_by_args, 
+        cmd_info =  {"name": cmd_name, "options_before_args": options_before_args, "keyword_optional": keyword_optional, "pass_by_args": pass_by_args, 
             "group": group, "func": func, "arguments": [], "options": [], "examples": [], "faqs": [], "hidden": False, "see_alsos": [],
             "kwgroup": kwgroup, "kwhelp": kwhelp, "help": help}
 
@@ -657,6 +657,52 @@ class Dispatcher():
 
         return value, tok
 
+    def parse_prop_op_value_list(self, tok, scanner):
+        # normal list of comma-separated tokens
+        values = []
+
+        while tok != None:
+            if tok.startswith("--"):
+                break
+                
+            value = self.process_prop_op_value(tok)
+            values.append(value)
+
+            tok = scanner.scan()
+            if tok != ",":
+                break
+
+            tok = scanner.scan()   # skip over the comma
+
+        return values, tok
+
+    def parse_int_list(self, tok, scanner, pipe_objects_enabled=True):
+        global pipe_object_list
+    
+        if "," in tok and not tok.startswith("--"):
+            # str_list in an option string
+            values = tok.split(",")
+            value = [int(v) for v in values]
+
+            tok = scanner.scan()        # skip over the whole string
+        else:
+            # normal list of comma-separated tokens
+            value = []
+    
+            while tok != None:
+                if tok.startswith("--"):
+                    break
+                    
+                value.append(int(tok))
+
+                tok = scanner.scan()
+                if tok != ",":
+                    break
+
+                tok = scanner.scan()   # skip over the comma
+
+        return value, tok
+
     def parse_tag_list(self, tok, scanner):
         tag_list = []
 
@@ -809,6 +855,10 @@ class Dispatcher():
                     value, tok = self.parse_num_list(tok, scanner)
                     if len(value)==0 and required:
                         self.syntax_error("missing value for required argument: " + arg_name)
+                elif arg_type == "int_list":
+                    value, tok = self.parse_int_list(tok, scanner)
+                    if len(value)==0 and required:
+                        self.syntax_error("missing value for required argument: " + arg_name)
                 elif arg_type == "tag_list":
                     value, tok = self.parse_tag_list(tok, scanner)
                     if len(value)==0 and required:
@@ -903,6 +953,9 @@ class Dispatcher():
             if not isinstance(value, list):
                 value = [value]
             value = self.convert_str_list_to_arg_dict(value)
+        elif opt_type == "int_list":
+            if not isinstance(value, list):
+                value = [value]
         elif opt_type == "num_list":
             if not isinstance(value, list):
                 value = [value]
@@ -1146,22 +1199,6 @@ class Dispatcher():
         
         # TODO: change to cmd_parts parsing, which naturally separates options cleanly (utils.cmd_split)
 
-        # add quotes that were removed by windows/shell command processor
-        # console.diag("before: args={}".format(args))
-        # self.add_quotes_to_string_args(args)
-        # console.diag("after: args={}".format(args))
-
-        # #add single quotes around option values without quotes
-        # args = []
-        # for arg in orig_args:
-        #     #if "=" in arg:
-        #     if arg.startswith("--") and "=" in arg:
-        #         pre, post = arg.split("=", 1)
-        #         if post and "=" in post and not post[0] in ['"', "'", "{"]:
-        #             arg = "{}='{}'".format(pre, post)
-        #     args.append(arg)
-        # console.diag("after: args={}".format(args))
-
         # be sure to reset this for each parse (for multi-command XT sessions)
         global explict_options
         explict_options = {}
@@ -1186,37 +1223,48 @@ class Dispatcher():
         console.diag("start of command parsing: {}".format(text))
 
         # process any options before the cmd as RAW options
-        raw_options = []
-        tok = self.collect_raw_options(raw_options, scanner, tok)
+        # raw_options = []
+        # tok = self.collect_raw_options(raw_options, scanner, tok)
 
         # process COMMAND keywords
         cmd_info, tok = self.get_cmd_info(tok, scanner)
         self.cmd_info = cmd_info
 
         if "kwgroup_name" in cmd_info:
-            # user type incomplete command - display appropriate help
-            if raise_syntax_exception:
-                errors.syntax_error("incomplete command")
+            cmd_info = get_command("help")
 
-            if command_help_func:
-                caller = self.impl_dict[command_help_func.__module__]
-                kwgroup_help_func(caller, cmd_info)
-                return
-            else:
-                errors.env_error("no registered 'help' command")
+        self.cmd_info = cmd_info
+
+            # # user type incomplete command - display appropriate help
+            # if raise_syntax_exception:
+            #     errors.syntax_error("incomplete command")
+
+            # if command_help_func:
+            #     # parse any help-specific options
+            #     help_options = {}
+            #     self.parse_options(help_options, options, scanner, tok)
+
+            #     caller = self.impl_dict[command_help_func.__module__]
+            #     kwgroup_help_func(caller, cmd_info)
+            #     return
+            # else:
+            #     errors.env_error("no registered 'help' command")
 
         cmd_name = cmd_info["name"]
         self.cmd_words = cmd_name.replace("_", " ")
         func = cmd_info["func"]
         options = cmd_info["options"]
         arguments = cmd_info["arguments"]
-
-        # build a dictionary of arguments and options to be passed
-        arg_dict = {}
+        options_before_args = cmd_info["options_before_args"]
 
         # command-specific help?
-        if "help" in raw_options:
-            help_value = raw_options["help"]
+        # if "help" in raw_options:
+        #     help_value = raw_options["help"]
+        #     if help_value != None:
+        #         self.syntax_error("unexpected text after '--help': " + help_value)
+        if tok == "--help":
+            help_value = scanner.scan()
+
             if help_value != None:
                 self.syntax_error("unexpected text after '--help': " + help_value)
 
@@ -1227,12 +1275,17 @@ class Dispatcher():
             command_help_func("help", caller, cmd_info)
             return
 
-        tok = self.process_arguments(scanner, tok, arguments, arg_dict)
+        # build a dictionary of arguments and options to be passed
+        arg_dict = {}
 
-        if not cmd_info["options_before_cmd"]:
-            tok = self.collect_raw_options(raw_options, scanner, tok)
-
-        self.process_raw_options(options, raw_options, arg_dict)
+        if options_before_args:
+            # options come before arguments
+            tok = self.parse_options(arg_dict, options, scanner, tok)
+            tok = self.process_arguments(scanner, tok, arguments, arg_dict)
+        else:
+            # arguments come before options
+            tok = self.process_arguments(scanner, tok, arguments, arg_dict)
+            tok = self.parse_options(arg_dict, options, scanner, tok)
 
         # there should be no remaining tokens
         if tok:
@@ -1266,3 +1319,105 @@ class Dispatcher():
             output = caller.set_capture_output(False)
 
         return output
+
+    def parse_options(self, arg_dict, options, scanner, tok):
+        '''
+        TODO: parse scanned option according to the *options* argument, 
+        not as "raw options".  
+        '''
+        raw_options = []
+
+        #tok = self.collect_raw_options(raw_options, scanner, tok)
+        #self.process_raw_options(options, raw_options, arg_dict)
+        tok = self.process_cmd_options(arg_dict, options, scanner, tok)
+
+        return tok
+
+    def process_cmd_options(self, arg_dict, options, scanner, tok):
+
+        while tok and tok.startswith("--"):
+            name = tok[2:]    # remove dashes
+            match = self.list_item_by_name(name, options)
+            if not match:
+                self.syntax_error("cmd '{}' doesn't have an option named: {}".format(self.cmd_words, tok))
+
+            name = match["name"]        # full name
+            tok = scanner.scan()        # skip over name
+            value = True
+            found_equals = False
+
+            if tok == "=":
+                found_equals = True
+                tok = scanner.scan()        # skip over equals
+
+            values = match["values"] if "values" in match else None
+            #value = self.process_option_value(name, match["type"], value, values)
+            value, tok = self.parse_option_value(name, match["type"], values, found_equals, tok, scanner)
+
+            arg_dict[name] = value
+
+        return tok
+
+    def parse_option_value(self, name, opt_type, keywords, found_equals, tok, scanner):
+
+        if opt_type == "str_list":
+            value, tok = self.parse_string_list(tok, scanner)
+            if len(value)==0 and required:
+                self.syntax_error("missing value for required option: " + name)
+
+        elif opt_type == "num_list":
+            value, tok = self.parse_num_list(tok, scanner)
+            if len(value)==0 and required:
+                self.syntax_error("missing value for required option: " + name)
+
+        elif opt_type == "int_list":
+            value, tok = self.parse_int_list(tok, scanner)
+            if len(value)==0 and required:
+                self.syntax_error("missing value for required option: " + name)
+
+        elif opt_type == "tag_list":
+            value, tok = self.parse_tag_list(tok, scanner)
+            if len(value)==0 and required:
+                self.syntax_error("missing value for required option: " + name)
+
+        elif opt_type == "prop_op_value":
+            value, tok = self.parse_prop_op_value_list(tok, scanner)
+            if len(value)==0 and required:
+                self.syntax_error("missing value for required option: " + name)
+
+        elif opt_type == "named_arg_list":
+            value, tok = self.parse_string_list(tok, scanner)
+            if not isinstance(value, list):
+                value = [value]
+            value = self.convert_str_list_to_arg_dict(value)
+
+        elif opt_type == "flag":
+            if found_equals:
+                value = tok
+                tok = scanner.scan()
+                value = self.parse_flag_value(name, value)
+            else:
+                value = 1
+        else:
+            # its a simple value
+            value = tok
+            tok = scanner.scan()
+
+            if opt_type == "int":
+                value = int(value)
+
+            elif opt_type == "float":
+                value = float(value)
+
+            elif opt_type == "bool":
+                value = value.lower()
+                if value in ["true", "1"]:
+                    value = True
+                elif value in ["false", "0"]:
+                    value = False
+                else:
+                    self.syntax_error("Illegal value for boolean option: " + str(value))
+            elif opt_type != "str":
+                errors.internal_error("Unsupported opt_type={}".format(opt_type))
+        
+        return value, tok
\ No newline at end of file
