@@ -591,6 +591,10 @@ class ReportBuilder():
                 elif value is not None:
                     # don't let None values influence the type of field
                     # assume value found is string-like
+                    
+                    # ensure value is a string
+                    value = str(value)
+
                     value_str = str_fmt.format(value) if value else ""
                     if first_value:
                         is_numeric = utils.str_is_float(value)
@@ -740,6 +744,9 @@ class ReportBuilder():
                         int_fmt = "{:" + align + str(col_width) + "d}"
                         text += int_fmt.format(value)
                     else:
+                        # ensure value is a string
+                        value = str(value)
+                        
                         str_fmt = "{:" + align + str(col_width)  + "." + str(col_width) + "s}"
                         #value = value if value else ""
                         value = "" if value is None else value
