@@ -27,7 +27,7 @@ FN_CHECKPOINT_WILD = "checkpoints/*"
 class Run():
 
     def __init__(self, config=None, store=None, xt_logging=True, aml_logging=True, checkpoints_enabled=True,
-        tensorboard_path=None):
+        tensorboard_path=None, supress_normal_output=False):
         ''' 
         this initializes an XT Run object so that ML apps can use XT services from within their app, including:
             - hyperparameter logging
@@ -44,6 +44,7 @@ class Run():
         self.xt_logging = False
         self.metric_report_count = 0
         self.metric_names = OrderedDict()
+        self.supress_normal_output = supress_normal_output
 
         # tensorboard writers
         self.train_writer = None
@@ -68,9 +69,6 @@ class Run():
 
             self.init_tensorboard()
 
-        # load azure libraries on demand
-        from .backends.backend_aml import AzureML
-
         self.ws_name = os.getenv("XT_WORKSPACE_NAME", None)
         self.exper_name = os.getenv("XT_EXPERIMENT_NAME", None)
         self.run_name = os.getenv("XT_RUN_NAME", None)
@@ -87,9 +85,11 @@ class Run():
 
                 context_dict = json.loads(json_context)
                 self.context = utils.dict_to_object(context_dict)
-                console.print("run context loaded: {}".format(self.context.run_name))
+                if not supress_normal_output:
+                    console.print("run context loaded: {}".format(self.context.run_name))
             else:
-                console.print("run context file not found: {}".format(fn_context))
+                if not supress_normal_output:
+                    console.print("run context file not found: {}".format(fn_context))
 
         mc = os.getenv("XT_MONGO_CONN_STR")
         self.mongo_conn_str = utils.base64_to_text(mc)
@@ -103,19 +103,27 @@ class Run():
         provider_code_path = os.getenv("XT_STORE_CODE_PATH")
 
         run_cache_dir = None
+        self.config = config
+
         if config:
             run_cache_dir = config.get("general", "run-cache-dir")
 
-        from azureml.core import Run as AmlRun
+        is_aml_run = bool(os.getenv("AML_WORKSPACE_NAME"))
+        
+        if is_aml_run:
+            # load azure libraries on demand
+            from .backends.backend_aml import AzureML
+            from azureml.core import Run as AmlRun
+
+            self.aml_run = AmlRun.get_context()     # assumes app is running under AML
+        else:
+            self.aml_run = None
 
         self.aml_logging = aml_logging
-        self.aml_run = AmlRun.get_context()     # assumes app is running under AML
         self.is_aml_child = False    
+        self.is_aml = False   # TODO: remove need for this since we now treat AML runs normally
 
-        # experiment: try treating AML runs no differently
-        self.is_aml = False   # hasattr(self.aml_run, "type")   # 
-
-        if self.run_name:
+        if True:    # self.run_name:
             if not self.store_creds and not config:
                 # if store_creds not set, this app is running outside of XT control
                 # provide access to XT store for dev/test purposes
@@ -126,7 +134,8 @@ class Run():
             self.store = Store(self.store_creds, provider_code_path=provider_code_path, run_cache_dir=run_cache_dir, 
                 mongo_conn_str=self.mongo_conn_str, config=config)
 
-            console.print("XT logging enabled: ", self.run_name)
+            # if not supress_normal_output:
+            #     console.print("XT logging enabled: ", self.run_name)
 
         # distributed training support
         self.rank = None
@@ -292,7 +301,8 @@ class Run():
 
             if metric_names_changed:
                 ddx = {"metric_names": list(self.metric_names)}
-                console.print("updating metric_names: {}".format(ddx))
+                # if not self.supress_normal_output:
+                #     console.print("updating metric_names: {}".format(ddx))
 
                 self.store.mongo.update_mongo_run_from_dict(self.ws_name, self.run_name, ddx)
 
@@ -399,7 +409,7 @@ class Run():
         path = "store://{}/{}".format(utils.make_share_name(share), store_path)
         return {"count": count, "path": path}
 
-    def get_next_hp_set_in_search(self, hp_space_dict, search_type=None):
+    def get_next_hp_set_in_search(self, hp_space_dict, search_type=None, hparam_search=None):
         '''
         args:
             hp_space_dict: a dict of HP name/space_text pairs (space_text specifies search space for HP)
@@ -420,11 +430,13 @@ class Run():
 
         from xtlib.hparams.hparam_search import HParamSearch
         
-        hparam_search = HParamSearch()
-        space_records = hparam_search.parse_hp_config_yaml(hp_space_dict)
-
         if not search_type:
             search_type = self.context.search_type
 
+        if not hparam_search:
+            hparam_search = HParamSearch()
+            
+        space_records = hparam_search.parse_hp_config_yaml(hp_space_dict, search_type)
+
         hp_dict = hparam_search.hp_search_core(self.context, search_type, self.store, run_name=self.run_name, space_records=space_records)
         return hp_dict
\ No newline at end of file
