@@ -11,8 +11,10 @@ from xtlib import utils
 from xtlib import errors
 from xtlib import constants
 from xtlib import file_utils
-from .console import console
-from .report_builder import ReportBuilder   
+from xtlib import job_helper
+
+from xtlib.console import console
+from xtlib.report_builder import ReportBuilder   
 from xtlib.storage import fixup_mongo_runs
 
 def expand_run_list(store, mongo, workspace, name_list):
@@ -154,7 +156,8 @@ def get_run_property_dicts():
         "node": "node_index", "outer": "is_outer", "parent": "is_parent", "path": "path", 
         "pool": "pool", "repeat": "repeat", "restarts": "restarts", "run": "run_name", "run_num": "run_num", 
         "script": "script", "search": "search_type", "search_style": "search_style", "service_type": "service_type", "sku": "sku",
-        "status": "status", "target": "compute", "username": "username", "vc": "vc", "workspace": "ws",
+        "status": "status", "target": "compute", "username": "username", "vc": "vc", "workspace": "ws", "xt_build": "xt_build", 
+        "xt_version": "xt_version",
         "hparams": "hparams", "metrics": "metrics", "tags": "tags",
 
          "ended": "end_time", "started": "start_time", "duration": "run_duration", "queued": "queue_duration",
@@ -198,6 +201,8 @@ def get_run_property_dicts():
         "username": "the login name of the user who submitted the run",
         "vc": "the name of the virtual cluster for the run",
         "workspace": "the name of the workspace containing the run",
+        "xt_build": "the build date of xt that was used to launch the run",
+        "xt_version": "the version of xt that was used to launch the run",
 
         "ended": "when the run completed",
         "started": "when the run started running",
@@ -213,11 +218,14 @@ def get_run_property_dicts():
 def build_filter_part(fd, args, arg_name, store_name):
     value = args[arg_name]
     if value:
-        fd[store_name] = value
+        if isinstance(value, list):
+            fd[store_name] = {"$in": value}
+        else:
+            fd[store_name] = value
 
 def build_run_filter_dict(run_list, user_to_actual, builder, args):
     fd = {}
-    option_filters = ["job", "experiment", "target", "service_type", "box", "status", "parent", "child", "outer"]  
+    option_filters = ["job", "experiment", "target", "service_type", "box", "status", "parent", "child", "outer", "username"]  
 
     if run_list:
         # filter by specified job names
@@ -346,28 +354,36 @@ def list_runs(store, config, args):
         else:
             console.print("no matching runs found")
 
-def get_run_records(store, workspace, run_names):
+def get_run_records(store, workspace, run_names, fields_dict=None):
     ''' return run records for specified run names'''
 
     mongo = store.get_mongo()
 
-    # get philly service_run_id for each run name
     filter_dict = {}
     filter_dict["run_name"] = {"$in": run_names}
 
-    fields_dict = {"cluster": 1, "service_run_id": 1, "vc": 1}
+    if not fields_dict:
+        # by default, get everything but the log records
+        fields_dict = {"log_records": 0}
 
     run_records = mongo.get_info_for_runs(workspace, filter_dict, fields_dict)
 
     return run_records
 
-def get_run_record(store, workspace, run_name):
-    run_records = get_run_records(store, workspace, [run_name])
+def get_run_record(store, workspace, run_name, fields_dict = None):
+    run_records = get_run_records(store, workspace, [run_name], fields_dict)
     if not run_records:
         errors.store_error("Run {} does not exist in workspace {}".format(run_name, workspace))
     rr = run_records[0]
     return rr
 
+def get_service_node_info(store, workspace, run_name):
+    rr = run_helper.get_run_record(store, workspace, name, {"job_id": 1, "node_index": 1})
+    job_id = rr["job_id"]
+    node_index = rr["node_index"]
+
+    service_node_info, backend = job_helper.get_service_node_info(job_id, node_index)
+
 def get_rightmost_run_num(run):
     if not run.startswith("run"):
         errors.syntax_error("Illegal run name, must start with 'run'")
@@ -497,17 +513,31 @@ def validate_run_name(store, ws, run_name, error_if_invalid=True, parse_only=Fal
                 return None, None, None
     return ws, run_name, ws + "/" + run_name
 
-
-
-def build_metrics_sets(records):
+def build_metrics_sets(records, steps=None, merge=False, metrics=None):
     '''
     builds a collection of metrics sets.  A metric set is a set of metrics that have been reported together 
     in the run log.  Here 'together' means as a single log entry.
 
-    'records' is the set of dict records of a run log.
+    args:
+        'records' is the set of dict records of a run log.
+        'steps' is an optional set of steps to filter by.
+        'merge': when True, merge all datasets into a single one
+        'metrics': list of specific metric names to extract
     '''
     # first step: put each metric into their own set (with time-stamped records)
     metric_sets_by_keys = {}
+    step_index = None
+    next_step = None
+    step_name = None
+    
+    # for merge
+    last_record = {}
+    last_step = None
+    merged_records = []
+
+    if steps:
+        step_index = 0
+        next_step = steps[0]
 
     for log_dict in records:
         if not log_dict:
@@ -517,23 +547,83 @@ def build_metrics_sets(records):
             continue
 
         dd = log_dict["data"]
-        keys = list(dd.keys())
-        keys.sort()
-        keys_str = json.dumps(keys)
+        
+        if step_name is None:
+            if constants.STEP_NAME in dd:
+                step_name = dd[constants.STEP_NAME]
+            elif "step" in dd:
+                step_name = "step"
+            elif "epoch" in dd:
+                step_name = "epoch"
 
-        if not keys_str in metric_sets_by_keys:
-             metric_sets_by_keys[keys_str] = []
+        if steps and step_name:
+            # filter this record (skip if neq next_step)
+            step_value = dd[step_name]
+
+            while step_value > next_step:
+                # compute next step
+                step_index += 1
+                if step_index < len(steps):
+                    next_step = steps[step_index]
+                else:
+                    # found all specified steps
+                    break
+
+            if step_value < next_step:
+                continue
 
         # add time to record
         dd[constants.TIME] = log_dict["time"]
 
-        metric_set = metric_sets_by_keys[keys_str]
-        metric_set.append(dd)
+        if metrics:
+            # only collect step_name and metrics in "metrics"
+            dd2 = {}
+            dd2[step_name] = dd[step_name]
+            for name, value in dd.items():
+                if name in metrics:
+                    dd2[name] = value
+
+            # store back into dd
+            dd = dd2
+
+        if merge:
+            if last_step == step_value:
+                # add to last_record
+                for name, value in dd.items():
+                    last_record[name] = value
+            else:
+                merged_records.append(dd)
+                last_record = dd
+                last_step = step_value
+        else:
+            # collect into multiple metric sets
+            keys = list(dd.keys())
+            #keys.sort()
+            keys_str = json.dumps(keys)
+
+            if not keys_str in metric_sets_by_keys:
+                metric_sets_by_keys[keys_str] = []
+
+            metric_set = metric_sets_by_keys[keys_str]
+            metric_set.append(dd)
 
     metric_sets = []
-    for keys_str, records in metric_sets_by_keys.items():
-        df = {"keys": json.loads(keys_str), "records": records}
+
+    if merge:
+        # build set of keys that covers all records
+        keys = {}
+        for dd in merged_records:
+            for name in dd:
+                keys[name] = 1
+
+        key_list = list(keys.keys())
+
+        df = {"keys": key_list, "records": merged_records}
         metric_sets.append(df)
+    else:
+        for keys_str, records in metric_sets_by_keys.items():
+            df = {"keys": json.loads(keys_str), "records": records}
+            metric_sets.append(df)
 
     return metric_sets
 
@@ -563,4 +653,54 @@ def get_client_cs(core, ws, run_name):
         cs = cs_plus["cs"]
         box_secret = cs_plus["box_secret"]
 
-    return cs, box_secret
\ No newline at end of file
+    return cs, box_secret
+
+def wrapup_runs_by_node(store, job_id, node_id, node_cancel_result):
+    '''
+    wrap up a run from azure batch.  run may have started, or may have completed.  
+    '''
+    # get job_record
+    job_info = job_helper.get_job_record(store, job_id)
+    ws_name = job_info["ws_name"]
+
+    # loads the controller's MRC for context of this node
+    text = store.read_job_file(job_id, constants.FN_MULTI_RUN_CONTEXT)
+    mrc_data = json.loads(text)
+
+    context_by_nodes = mrc_data["context_by_nodes"]
+    context_plus = context_by_nodes[node_id]
+    context = context_plus["runs"][0]
+    
+    # get runs that need wrapup in job
+    node_index = utils.node_index(node_id)
+
+    unwrapped = ["created", "queued", "spawning", "allocating", "running"]
+    filter_dict = {"job_id": job_id, "node_index": node_index, "status":  {"$in": unwrapped}}
+    fields_dict = {"log_records": 0}
+
+    runs = store.mongo.get_info_for_runs(ws_name, filter_dict, fields_dict)
+
+    for run in runs:
+        wrapup_run_with_context(store, run, context)
+
+    return runs
+
+def wrapup_run_with_context(store, run, context_dict):
+    context = utils.dict_to_object(context_dict)
+    status = "cancelled"
+    exit_code = 0
+    node_id = utils.node_id(context.node_index)
+
+    # use info from run, when possible (context is shared among all child runs)
+    run_index = run["run_index"]
+    run_name = run["run_name"]
+
+    # these we don't have info for
+    rundir = None    # unknown
+    log = True
+    capture = True
+
+    store.wrapup_run(context.ws, run_name, context.aggregate_dest, context.dest_name, 
+        status, exit_code, context.primary_metric, context.maximize_metric, context.report_rollup, 
+        rundir, context.after_files_list, log, capture, job_id=context.job_id, node_id=node_id,
+        run_index=run_index)    
\ No newline at end of file
