@@ -382,17 +382,17 @@ class Runner():
         else:
             self.store.create_job_file(job_id, target_name, sweeps_text)
 
-    def attach_if_needed(self, workspace, run_data_list_by_box, escape, attach):
-        # ATTACH or provide attach cmd
-        first_run = run_data_list_by_box[0][0]   
+    # def attach_if_needed(self, workspace, run_data_list_by_box, escape, attach):
+    #     # ATTACH or provide attach cmd
+    #     first_run = run_data_list_by_box[0][0]   
 
-        if attach:
-            time.sleep(1)
-            self.core.client.monitor_attach_run(workspace, first_run["run_name"], show_waiting_msg=False, escape=escape)
-        # else:
-        #     box_name = first_run["box_name"]
-        #     full_run_name = workspace + "/" + first_run["run_name"].upper()   
-        #     console.print("To view output: xt attach {}".format(full_run_name))
+    #     if attach:
+    #         time.sleep(1)
+    #         self.core.client.monitor_attach_run(workspace, first_run["run_name"], show_waiting_msg=False, escape=escape)
+    #     # else:
+    #     #     box_name = first_run["box_name"]
+    #     #     full_run_name = workspace + "/" + first_run["run_name"].upper()   
+    #     #     console.print("To view output: xt attach {}".format(full_run_name))
 
     def build_runs_by_box(self, job_runs, workspace):
         # build box_name => runs dict for job info file
@@ -582,8 +582,8 @@ class Runner():
             self.process_args(args)
 
         # create backend helper (pool, philly, batch, aml)
-        cluster = args["cluster"]
-        vc = args["vc"]
+        cluster = utils.safe_value(compute_def, "cluster")
+        vc = utils.safe_value(compute_def, "vc")
         self.backend = self.core.create_backend(compute, cluster=cluster, vc=vc, username=None)
 
         # add conda_packages and pip_packages from SETUP to ARGS
@@ -739,6 +739,11 @@ class Runner():
             data_local = os.path.realpath(data_local.replace("$scriptdir", script_dir))
             args["data_local"] = data_local
 
+        model_local = args["model_local"]
+        if "$scriptdir" in model_local:
+            model_local = os.path.realpath(model_local.replace("$scriptdir", script_dir))
+            args["model_local"] = model_local
+
         # ADJUST CMDS: this allows backend to write scripts to snapshot dir, if needed, as a way of adjusting/wrapping run commands
         self.backend.adjust_run_commands(job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, args=args)
 
@@ -820,11 +825,7 @@ class Runner():
 
             self.store.log_job_info(job_id, dd)
 
-        # process ATTACH flag
         escape = args["escape"]
-        attach = args["attach"]
-
-        self.attach_if_needed(workspace, job_runs, escape, attach)
 
         # remove temp files
         # this is causing a permission denied error currently (due to Azure holding it open?)
@@ -837,29 +838,9 @@ class Runner():
         xtd["last_job"] = job_id
 
         xt_dict.write_xt_dict(xtd)
-        
+
         # return values for API support (X)
-        return cmds, run_specs, using_hp, using_aml_hparam, sweeps_text, pool_info 
-
-    # def build_static_runs_by_node(self, total_runs, total_nodes):
-    #     if total_runs < total_nodes:
-    #         errors.combo_error("total runs={} must be >= total_nodes={}".format(total_runs, total_nodes))
-
-    #     rbn = {}
-    #     full_count = math.ceil(total_runs/total_nodes)
-    #     runs = total_runs
-
-    #     for i in range(total_nodes):
-    #         start_index = i * full_count
-    #         node_runs = runs if runs < full_count else full_count
-
-    #         dd = {"runs_remaining": node_runs, "start_index": start_index, "run_count": node_runs}
-
-    #         node_id = utils.node_id(i)
-    #         rbn[node_id] = dd
-    #         runs -= full_count
-
-    #     return rbn
+        return cmds, run_specs, using_hp, using_aml_hparam, sweeps_text, pool_info, job_id 
 
     def distribute_cmds_to_nodes(self, cmds, num_nodes):
         cmds_by_node = {}
