@@ -37,6 +37,9 @@ class MongoDB():
         self.mongo_client = None
         self.mongo_db = None
 
+        # keep a count of how many retryable errors we have encountered
+        self.retry_errors = 0
+
         self.run_cache_dir = os.path.expanduser(run_cache_dir) if run_cache_dir else None
 
         # initialize mondo-db now
@@ -77,14 +80,18 @@ class MongoDB():
         # remove associated mongo_db container
         container = self.mongo_db[ws_name]
         container.drop()
-        #console.print("  mongo_db container dropped=", container)
+        count = container.count()
+
+        console.diag("  after mongo_db container={} dropped, count={}=".format(container, count))
 
         # remove counters for this workspace
-        self.mongo_db.ws_counters.remove( {"_id": ws_name} )
+        cmd = lambda: self.mongo_db.ws_counters.remove( {"_id": ws_name} )
+        self.mongo_with_retries("remove_workspace", cmd, ignore_error=True)
 
-        # remove counters for this workspace
+        # remove legacy counters for this workspace
         end_id = ws_name + "-end_id"
-        self.mongo_db.ws_counters.remove( {"_id": end_id} )
+        cmd = lambda: self.mongo_db.ws_counters.remove( {"_id": end_id} )
+        self.mongo_with_retries("remove_workspace", cmd)
 
     def remove_cache(self, ws_name):
         # remove appropriate node of run_cache_dir
@@ -96,15 +103,18 @@ class MongoDB():
             console.print("  zapping cache_dir=", cache_dir)
             file_utils.zap_dir(cache_dir)
 
-    def get_next_sequential_job_id(self, ws_name, path, default_next):
+    def get_next_sequential_job_id(self, default_next):
+        job_ws = "__jobs__"
+        path = "next_job"
+
         db = self.mongo_db
 
         # does a counters doc exit for this ws_name?
-        cursor = db.ws_counters.find({"_id": ws_name}).limit(1)
+        cursor = db.ws_counters.find({"_id": job_ws}).limit(1)
         if not cursor.count():
-            db.ws_counters.insert_one( {"_id": ws_name, path: default_next} )
+            db.ws_counters.insert_one( {"_id": job_ws, path: default_next} )
 
-        document = db.ws_counters.find_and_modify( {"_id": ws_name}, update={"$inc": {path: 1} }, new=False)
+        document = db.ws_counters.find_and_modify( {"_id": job_ws}, update={"$inc": {path: 1} }, new=False)
         next_id = document[path]
 
         return next_id
@@ -118,9 +128,13 @@ class MongoDB():
 
     def get_next_sequential_ws_id(self, ws_name, path, default_next_run):
         db = self.mongo_db
+
+        assert not "/" in ws_name 
+        assert not "/" in path 
+        
         console.diag("ws={}, path={}, default_next_run={}".format(ws_name, path, default_next_run))
 
-        # does a counters doc exit for this ws_name?
+        # does a counters doc exist for this ws_name?
         cursor = db.ws_counters.find({"_id": ws_name}).limit(1)
         if not cursor.count():
             console.diag("LEGACY ws={} found in get_next_sequential_ws_id".format(ws_name))
@@ -143,7 +157,7 @@ class MongoDB():
         return next_id
 
     def get_next_job_id(self, default_next=1):
-        return self.get_next_sequential_job_id("__jobs__", "next_job", default_next)
+        return self.get_next_sequential_job_id(default_next)
 
     def get_next_run_id(self, ws_name, default_next=1):
         return self.get_next_sequential_ws_id(ws_name, "next_run", default_next)
@@ -165,7 +179,10 @@ class MongoDB():
                 break
             # watch out for these exceptions: AutoReconnect, OperationFailure (and ???)
             except BaseException as ex:   # pymongo.errors.OperationFailure as ex:
-                logger.exception("Error in mongo_with_retries, ex={}".format(ex))
+                
+                # since we cannot config logger to supress stderr, don't log this
+                #logger.exception("Error in mongo_with_retries, ex={}".format(ex))
+                
                 # pymongo.errors.OperationFailure: Message: {"Errors":["Request rate is large"]}
                 if ignore_error:
                     console.print("ignoring mongo-db error: name={}, ex={}".format(name, ex))
@@ -180,9 +197,16 @@ class MongoDB():
                 # [1,61] so we don't die with a hard failure here
                 if i == 0:
                     backoff = 1 + 10*np.random.random()
+                    self.retry_errors += 1
                 else:
                     backoff = 1 + 60*np.random.random()
-                console.print("retrying mongo-db: name={}, retry={}/{}, backoff={:.2f}, ex={}".format(name, i+1, retry_count, backoff, ex))
+
+                ex_code = ex.code if hasattr(ex, "code") else ""
+                ex_msg = str(ex)[0:60]+"..."
+
+                console.print("retrying mongo-db: name={}, retry={}/{}, backoff={:.2f}, ex.code={}, ex.msg={}".format(name, i+1, retry_count, backoff, 
+                    ex_code, ex_msg))
+                    
                 time.sleep(backoff)
                 
         return result
@@ -195,8 +219,11 @@ class MongoDB():
         run_doc = copy.deepcopy(dd)
 
         # zap a few we don't want
-        del run_doc["event"]
-        del run_doc["time"]
+        if "event" in run_doc:
+            del run_doc["event"]
+
+        if "time" in run_doc:
+            del run_doc["time"]
 
         # add some new ones
         is_azure = utils.is_azure_batch_box(dd["box_name"])
@@ -215,7 +242,7 @@ class MongoDB():
 
         # first, add log record to ws/run document
         update_doc = { "$push": {"log_records": log_record} }
-        self.mongo_with_retries("add_run_event", lambda: self.mongo_db[ws_name].update_one( {"_id": run_name}, update_doc) )
+        self.mongo_with_retries("add_run_event", lambda: self.mongo_db[ws_name].update_one( {"_id": run_name}, update_doc, upsert=True) )
 
         event_name = log_record["event"]
         data_dict = log_record["data"]
@@ -283,7 +310,6 @@ class MongoDB():
         #console.print("update_mongo_run_from_dict: ws_name={}, run_name={}, update_doc={}".format(ws_name, run_name, update_doc))
 
         # do a REPLACE DOC update operation
-        #self.mongo_db[ws_name].update_one( {"_id": run_name}, update_doc)
         self.mongo_with_retries("update_mongo_run_from_dict", lambda: self.mongo_db[ws_name].update_one( {"_id": run_name}, update_doc) )
 
     def get_info_for_runs(self, ws_name, filter_dict, fields_dict=None):
@@ -460,8 +486,9 @@ class MongoDB():
     def does_jobs_exist(self):
         # does document for ws_name exist?
         job_id = "job1"
-        cursor = self.mongo_db["__jobs__"].find({"_id": job_id}, {"_id": 1}).limit(1)
-        found = cursor and cursor.count
+        cmd = lambda: self.mongo_db["__jobs__"].find({"_id": job_id}, {"_id": 1}).limit(1)
+        cursor = self.mongo_with_retries("does_jobs_exist", cmd)
+        found = bool(cursor and cursor.count)
 
         return found
 
@@ -501,7 +528,9 @@ class MongoDB():
         ws_name = None
 
         # does document for ws_name exist?
-        cursor = self.mongo_db["__jobs__"].find({"_id": job_id}, {"ws_name": 1}).limit(1)
+        cmd = lambda: self.mongo_db["__jobs__"].find({"_id": job_id}, {"ws_name": 1}).limit(1)
+        cursor = self.mongo_with_retries("get_job_workspace", cmd)
+
         if cursor and cursor.count():
             result = list(cursor)[0]
             if "ws_name" in result:
@@ -515,8 +544,11 @@ class MongoDB():
             - increment the job's "running_nodes" property
             - set the "job_status" property to "running"
         '''
-        self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$inc": {"running_nodes": 1} })
-        self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$set": {"job_status": "running"} })
+        cmd = lambda: self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$inc": {"running_nodes": 1} })
+        self.mongo_with_retries("job_node_start", cmd)
+
+        cmd = lambda: self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$set": {"job_status": "running"} })
+        self.mongo_with_retries("job_node_start", cmd)
 
     def job_node_exit(self, job_id):
         '''
@@ -524,171 +556,24 @@ class MongoDB():
             - decrement the job's "running_nodes" property 
             - if running_nodes==0, set the "job_status" property to "completed"
         '''
-        self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$inc": {"running_nodes": -1} })
-        self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id, "running_nodes": 0}, update={"$set": {"job_status": "completed"} })
+        cmd = lambda: self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$inc": {"running_nodes": -1} })
+        self.mongo_with_retries("job_node_exit", cmd)
 
-    # def get_next_run_index(self, job_id, node_id, first_call):
-    #     '''
-    #     args:
-    #         - job_id: job that requested the new run belongs to
-    #         - node_id: the id (e.g., node3) that is requesting the run index
-    #         - run_name: the name of the new run (e.g., run23.3)
-    #         - first_call: True if this is the first call the controller has made to this API
-
-    #     goals:
-    #         - support 2 schedule types:
-    #             - static: pre-allocate runs to nodes (based on number of nodes and runs)
-    #             - dynamic: allocate runs to nodes as the nodes become available
-    #         - correctly handle node restarts (so that node will re-run those runs it was working on)
-
-    #     processing:
-    #         - if first_call and node already has run_indexes, mark all as "waiting_for_restart"
-    #         - if node has a "waiting_for_restart" run index, that run index record is marked as "restarted" and returned  
-    #         - otherwise, uses the job schedule property to get the static or dynamic run index 
-    #           and create a new run index record (marked as "running")
-    #         - updates the list of run_indexes for the node (mongo db)
-
-    #     locking:
-    #         - the decrement of runs_remaining property is lock-protected by mongodb (for dynamic schedule)
-    #         - TODO: update node_index info in a lock-safe (atomic) way
-
-    #     returns:
-    #         - the selected run index record, or None (if no more runs for )
-    #     '''
-    #     entry = None
-
-    #     # status values
-    #     WAITING = "waiting_for_restart"
-    #     RESTARTED = "restarted"
-    #     RUNNING = "running"
-
-    #     run_indexes = self.get_run_indexes_by_node(job_id, node_id)
-        
-    #     if run_indexes:
-    #         if first_call:
-    #             # mark all run_idexes as "waiting_for_restart"
-    #             for ri in run_indexes:
-    #                 ri["status"] = WAITING
-
-    #         # is there a "waiting_for_restart" entry?
-    #         entry = next((ri for ri in run_indexes if ri["status"] == WAITING), None)
-    #         if entry:
-    #             entry["status"] = RESTARTED
-        
-    #     if entry is None:
-    #         # there were no "waiting for restart" runs for this node
-    #         # get next run index, as per dynamic/static schedule
-    #         cursor = self.mongo_db["__jobs__"].find( {"_id": job_id}, {"schedule": 1})
-    #         job_dd = list(cursor)[0] if cursor else None
-    #         schedule = utils.safe_value(job_dd, "schedule")
-    #         console.print("schedule={}, node={}".format(schedule, node_id))
-    #         run_index = None
-
-    #         if schedule == "static":
-    #             # determine run_index by node (runs per node are fixed)
-    #             #utils.debug_break()
-                
-    #             basekey = "static_runs_by_node.{}".format(node_id)
-    #             key = basekey + ".runs_remaining"
-
-    #             dd = self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$inc": {key: -1}}, new=True)
-    #             console.print("node={}, key={}, runs_remaining={}".format(node_id,  key, dd["static_runs_by_node"][node_id]["runs_remaining"]))
-
-    #             if dd:
-    #                 rbn = dd["static_runs_by_node"]
-    #                 info = rbn[node_id]
-    #                 prev_rr = info["runs_remaining"]
-
-    #                 if prev_rr >= 0:
-    #                     run_count = info["run_count"] 
-    #                     start_index = info["start_index"] 
-    #                     run_index = start_index + (run_count - (prev_rr + 1))
-    #                     console.print("returning node.run_index=", run_index)
-    #         else:
-    #             # determine run_index from job property "dynamic_runs_remaining"
-    #             dd = self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$inc": {"dynamic_runs_remaining": -1} })
-    #             if dd:
-    #                 prev_rr = dd["dynamic_runs_remaining"] 
-
-    #                 if prev_rr > 0:
-    #                     run_count = dd["run_count"] 
-    #                     run_index = run_count - prev_rr
-
-    #         if run_index is not None:
-    #             entry = {"run_index": run_index, "status": RUNNING}
-
-    #             # add entry to run_index for this node
-    #             if not run_indexes:
-    #                 run_indexes = []
-    #             run_indexes.append(entry)
-
-    #     # no matter what happened, we always update the list of run_index records
-    #     self.update_run_indexes_by_node(job_id, node_id, run_indexes)
-
-    #     return dict(entry) if entry else None
-
-    # def get_run_indexes_by_node(self, job_id, node_id):
-    #     run_indexes = None
-
-    #     filter_dict = {"_id": job_id}
-    #     fields_dict = {"run_indexes_by_node": 1}
-
-    #     cursor = self.mongo_with_retries("get_next_run_index", lambda: self.mongo_db["__jobs__"].find(filter_dict, fields_dict))
-    #     runs = list(cursor) if cursor else None
-
-    #     if runs:
-    #         run = runs[0]
-    #         run_indexes_by_node = utils.safe_value(run, "run_indexes_by_node")
-    #         run_indexes = utils.safe_value(run_indexes_by_node, node_id)
-
-    #     return run_indexes
-
-    # def update_run_indexes_by_node(self, job_id, node_id, run_indexes):
-    #     key = "run_indexes_by_node." + node_id
-    #     update_doc = { "$set": {key: run_indexes} }
-    #     console.print("update_run_indexes_by_node: job={}, node={}, update_doc={}".format(job_id, node_id, update_doc))
-
-    #     self.mongo_with_retries("update_run_indexes_by_node", lambda: self.mongo_db["__jobs__"].update_one( {"_id": job_id}, update_doc) )
+        cmd = lambda: self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id, "running_nodes": 0}, update={"$set": {"job_status": "completed"} })
+        self.mongo_with_retries("job_node_exit", cmd)
 
     def update_connect_info_by_node(self, job_id, node_id, connect_info):
         key = "connect_info_by_node." + node_id
         update_doc = { "$set": {key: connect_info} }
         self.mongo_with_retries("update_connect_info_by_node", lambda: self.mongo_db["__jobs__"].update_one( {"_id": job_id}, update_doc) )
 
-    # def mark_node_run_index_complete(self, job_id, node_id, run_index):
-    #     '''
-    #     args:
-    #         - job_id: job that owns the run
-    #         - node_id: computing node id (e.g., node3) that run in running on
-    #         - run_index: the 0-index of the job's runs
-
-    #     proressing:
-    #         - the specified run has completed, so remove it from the node's run_index
-    #           records
-    #     '''
-        
-    #     run_indexes = self.get_run_indexes_by_node(job_id, node_id)
-    #     removed = False
-    #     console.print("mark_node_run_index_complete: job_id={}, node_id={}, run_index={}, run_indexes={}".format(job_id, node_id, run_index, run_indexes))
-
-    #     for ri in run_indexes:
-    #         if ri["run_index"] == run_index:
-    #             run_indexes.remove(ri)
-    #             removed = True
-    #             break
-
-    #     if not removed:
-    #         errors.InternalError("mark_node_run_index_complete: entry not found in node's run_index list: " +
-    #             "job={}, node={}, run_index={}".format(job_id, node_id, run_index))
-
-    #     self.update_run_indexes_by_node(job_id, node_id, run_indexes)
-
     def job_run_start(self, job_id):
         '''
         A job's run has started running.  We need to:
             - increment the job's "running_runs" property 
         '''
-        self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$inc": {"running_runs": 1} })
+        cmd = lambda: self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$inc": {"running_runs": 1} })
+        self.mongo_with_retries("job_run_start", cmd)
 
     def job_run_exit(self, job_id, exit_code):
         '''
@@ -698,7 +583,8 @@ class MongoDB():
             - if exit_code != 0, increment the job's "error_runs" property
         '''
         error_inc = 1 if exit_code else 0
-        self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$inc": {"running_runs": -1, "completed_runs": 1, "error_runs": error_inc} })
+        cmd = lambda: self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$inc": {"running_runs": -1, "completed_runs": 1, "error_runs": error_inc} })
+        self.mongo_with_retries("job_run_exit", cmd)
 
     def run_start(self, ws_name, run_name):
         '''
@@ -710,7 +596,9 @@ class MongoDB():
         now_str = str(now)
 
         # get create_time of run
-        cursor = self.mongo_db[ws_name].find({"_id": run_name}, {"create_time": 1})
+        cmd = lambda: self.mongo_db[ws_name].find({"_id": run_name}, {"create_time": 1})
+        cursor = self.mongo_with_retries("run_start", cmd)
+
         doc = cursor.next()
         if doc and "create_time" in doc:
             create_time_str = doc["create_time"]
@@ -719,7 +607,8 @@ class MongoDB():
             # compute time in "queue" 
             queue_duration = utils.time_diff(now, create_time)
 
-            self.mongo_db[ws_name].find_and_modify( {"_id": run_name}, update={"$set": {"start_time": now_str, "queue_duration": queue_duration} })
+            cmd = lambda: self.mongo_db[ws_name].find_and_modify( {"_id": run_name}, update={"$set": {"start_time": now_str, "queue_duration": queue_duration} })
+            self.mongo_with_retries("run_start", cmd)
 
     def run_exit(self, ws_name, run_name):
         '''
@@ -730,7 +619,9 @@ class MongoDB():
         now_str = str(now)
 
         # get start_time of run
-        cursor = self.mongo_db[ws_name].find({"_id": run_name}, {"start_time": 1})
+        cmd = lambda: self.mongo_db[ws_name].find({"_id": run_name}, {"start_time": 1})
+        cursor = self.mongo_with_retries("run_exit", cmd)
+
         doc = cursor.next()
         if doc and "start_time" in doc:
             start_time_str = doc["start_time"]
@@ -739,4 +630,5 @@ class MongoDB():
             # compute run_duration 
             run_duration = utils.time_diff(now, start_time)
 
-            self.mongo_db[ws_name].find_and_modify( {"_id": run_name}, update={"$set": {"run_duration": run_duration} })
\ No newline at end of file
+            cmd = lambda: self.mongo_db[ws_name].find_and_modify( {"_id": run_name}, update={"$set": {"run_duration": run_duration} })
+            self.mongo_with_retries("run_exit", cmd)
