@@ -61,7 +61,7 @@ class Store():
         >>> store = Store(config=my_config)
         >>> run_names = store.get_run_names("ws1")
 
-        '''
+    '''
     
     def __init__(self, storage_creds=None, config=None, max_retries=10, run_cache_dir=None, feedback_enabled=True,  mongo_conn_str=None, 
         provider_code_path=None, validate_creds=False):
@@ -508,6 +508,8 @@ class Store():
             "compute": compute, "service_type": service_type, "sku": sku, "search_style": search_style,
             "search_type": search_type, "xt_version": constants.BUILD, "run_index": run_index}
 
+        self.mongo.create_mongo_run(dd)
+
         # log "created" event for child run
         self.log_run_event(ws_name, child_name, "created", dd, is_aml=is_aml)
 
@@ -528,8 +530,6 @@ class Store():
         # finally, add a "child_created" record to the parent
         self.log_run_event(ws_name, parent_run_name, "child_created", {"child_name": child_name}, is_aml=is_aml)
         
-        self.mongo.create_mongo_run(dd)
-
         return child_name
 
     def start_run(self, ws_name, exper_name=None, description=None, username=None,
@@ -562,7 +562,6 @@ class Store():
         ip = pc_utils.get_ip_address()
         hostname = pc_utils.get_hostname()
         is_aml = (aml_run_id is not None)
-        from_user = "zzz"
 
         # always log the true name of the box (since there can be multiple clients which would otherwise produce multiple "local"s)
         if box_name == "local":
@@ -579,16 +578,22 @@ class Store():
         script = os.path.basename(path) if path else None
         create_time = utils.get_time()
 
+        build_parts = constants.BUILD.split(",")
+        xt_version = build_parts[0].split(":")[1].strip()
+        xt_build = build_parts[1].split(":")[1].strip()
+
         dd = {"ws": ws_name, "run_name": run_name, "run_num": run_num, "run_guid": run_guid, "description": description, "exper_name": exper_name, 
             "job_id": job_id, "node_index": node_index, "is_outer": True, "is_parent": is_parent, "is_child": False, 
             "from_ip": ip, "from_computer_name": hostname, "username": username, 
             "box_name": box_name, "app_name": app_name, "repeat": repeat, "path": path, "script": script, "create_time": create_time,
             "node": node_index, "compute": compute, "service_type": service_type, "search_type": search_type, "sku": sku,
-            "xt_version": constants.BUILD, "search_style": search_style, "run_index": None}
+            "xt_build": xt_build, "xt_version": xt_version, "search_style": search_style, "run_index": None}
 
         if aml_run_id:
             dd["aml_run_id"] = aml_run_id
 
+        self.mongo.create_mongo_run(dd)
+
         self.log_run_event(ws_name, run_name, "created", dd, is_aml=is_aml)
 
         console.diag("start_run: after log_run_event")
@@ -602,7 +607,6 @@ class Store():
         if not is_aml:
             self.append_run_file(ws_name, run_name, constants.RUN_SUMMARY_LOG, text)
 
-        self.mongo.create_mongo_run(dd)
         console.diag("start_run: after append_workspace_file")
 
         return run_name
@@ -797,7 +801,7 @@ class Store():
         report_rollup, rundir, after_files_list, log_events=True, capture_files=True, job_id=None, is_parent=False, 
         after_omit_list=None, node_id=None, run_index=None):
 
-        console.print("wrapup_run: rundir=", rundir, ", exit_code=", exit_code)
+        #console.print("wrapup_run: rundir=", rundir, ", exit_code=", exit_code)
 
         if log_events:  
             # LOG "ENDED" to run_log, APPEND TO ALLRUNS
@@ -922,6 +926,9 @@ class Store():
     def create_job(self):
         # read legacy next_job_info in case we are first to run w/mongo-sequence numbers
         default_next = self.helper.read_legacy_next_job_id()
+        if not default_next:
+            default_next = 1
+            
         job_id = self.mongo.get_next_job_id(default_next=default_next)
 
         job_name = "job{}".format(job_id)
@@ -1048,13 +1055,13 @@ class Store():
 
     # ---- CAPTURE OUTPUT ----
 
-    def capture_stdout(self, fn=RUN_STDOUT):
-        self.cap_stdout = StreamCapture(sys.stdout, fn)
-        sys.stdout = self.cap_stdout
+    # def capture_stdout(self, fn=RUN_STDOUT):
+    #     self.cap_stdout = StreamCapture(sys.stdout, fn)
+    #     sys.stdout = self.cap_stdout
 
-    def capture_stderr(self, fn=RUN_STDERR):
-        self.cap_stderr = StreamCapture(sys.stderr, fn)
-        sys.stderr = self.cap_stderr
+    # def capture_stderr(self, fn=RUN_STDERR):
+    #     self.cap_stderr = StreamCapture(sys.stderr, fn)
+    #     sys.stderr = self.cap_stderr
 
     def release_stdout(self):
         if self.cap_stdout:
