@@ -120,7 +120,13 @@ class StoreBlobObjs():
 
     def _create_info_container_if_needed(self):
         info_container = constants.INFO_CONTAINER
-        if not self.provider.does_blob_exist(info_container, constants.STORAGE_INFO_FILE):
+
+        container_exists = self.provider.does_container_exist(info_container)
+        if not container_exists or not self.provider.does_blob_exist(info_container, constants.STORAGE_INFO_FILE):
+
+            # create container
+            if not container_exists:
+                self.provider.create_container(info_container)
 
             # create storage info file
             info = {"version": constants.STORAGE_VERSION, "paired_mongo": None}
@@ -179,7 +185,11 @@ class StoreBlobObjs():
 
         if not self.does_workspace_exist(ws_name):
             # avoid 10 retries and unfriendly storage errors
-            errors.store_error("Container doesn't exist: " + ws_name)
+            errors.store_error("container doesn't exist: " + ws_name)
+
+        if not self.provider.does_blob_exist(ws_name, blob_path):
+            # avoid 10 retries and unfriendly storage errors
+            errors.store_error("blob doesn't exist: container={}, path={}".format(ws_name, blob_path))
 
         blob_text = self.provider.get_blob_text(ws_name, blob_path)
         return blob_text
@@ -798,9 +808,11 @@ class StoreBlobObjs():
         # read next job number from control file
         info_dir = constants.INFO_DIR
         fn_next = info_dir + "/" + constants.JOBS_NEXT
+        next_id = None
 
-        text = self._read_blob(constants.INFO_CONTAINER, fn_next)
-        next_id = int(text) if text else 1
+        if self.provider.does_blob_exist(constants.INFO_CONTAINER, fn_next):
+            text = self._read_blob(constants.INFO_CONTAINER, fn_next)
+            next_id = int(text) if text else 1
         return next_id
 
     #---- JOBS ----
