@@ -119,7 +119,7 @@ def main(cmd=None, new_start_time=None, capture_output=False, mini=False, raise_
             "cancel_job": ["job"], 
             "clear_credentials": [],
             "config_cmd": ["default", "create"],
-            "run": ["script", "script-args", "experiment", "hp-config", "max-runs", "nodes", "runs", "search-type"], 
+            "run": ["script", "script-args", "experiment", "hp-config", "max-runs", "nodes", "runs", "search-type", "target"], 
             "extract": ["run-name", "dest-dir", "browse"], 
             "list_runs": ["run-list", "job", "experiment", "all", "first", "last", "filter", "sort", "reverse", "status", "available"],
             "list_jobs": ["job-list", "experiment", "all", "first", "last", "filter", "sort", "reverse", "status", "available"],
@@ -128,7 +128,11 @@ def main(cmd=None, new_start_time=None, capture_output=False, mini=False, raise_
                 "legend-titles", "marker-shape", "marker-size", "max-runs", "max-traces", "plot-titles", "save-to", "show-legend",
                 "smoothing-factor", "title", "x"],  
             "help": ["command", "about", "browse", "version"],
+            "help_topics": ["topic", "browse"],
             "generate_help": ["dest-dir"],
+            "view_console": ["target", "workspace", "node-index"],
+            "create_demo": ["destination", "response"],
+            "create_services_template": []
             #"root": ["help", "console", "stack-trace"]
             }
 
@@ -138,7 +142,7 @@ def main(cmd=None, new_start_time=None, capture_output=False, mini=False, raise_
 
     
     # hide under-development commands
-    hide_commands  = ["collect_logs", "start_tensorboard", "stop_tensorboard"]
+    hide_commands  = ["collect_logs", "start_tensorboard", "stop_tensorboard", "zip", "unzip", "wget"]
 
     # hide internal cmds (for xt development use only)
     hide_commands.append("generate_help")
