@@ -28,50 +28,13 @@ class XTVault():
 
     def init_creds(self, authentication):
         cache_client = CacheClient()
-        auth_needed = True
+        loaded = False
         self.authentication = authentication
 
-        fn_keys = "keys.bin"
-        if os.path.exists(fn_keys):
-            # GROK server
-            creds = file_utils.read_text_file(fn_keys)  
-            self.apply_creds(creds)
-            console.print("init_creds: using grok server 'keys.bin' file")
-            auth_needed = False
-        else:
-            sc = os.getenv("XT_STORE_CREDS")
-            sc = utils.base64_to_text(sc)
-
-            mc = os.getenv("XT_MONGO_CONN_STR")
-            mc = utils.base64_to_text(mc)
-            #print("init_cred: sc={}, mc={}".format(sc, mc))
-
-            if sc and mc:
-                # XT client on compute node 
-
-                # cleanup (for testing from client)
-                sc = sc.replace('\\"', '"')
-                mc = mc.replace('\\"', '"')
-                if mc.startswith('"'):
-                    mc = mc[1:-1]
-
-                # print("sc=", sc)
-                sc_data = json.loads(sc)
-                store_name = sc_data["name"]
-                store_key = sc_data["key"]
-
-                # sample mc: mongodb://xt-sandbox-cosmos:kBOWLQrseZ
-                prefix, mc_rest = mc.split("://", 1)
-                mc_name, _ = mc_rest.split(":", 1)
-
-                # creds are limited in this case to just Store access [storage + mongo]
-                creds = json.dumps({store_name: store_key, mc_name: mc})
-                self.apply_creds(creds)
-    
-                console.print("init_creds: using compute node ENV VAR settings for store={}, mongo={}".format(store_name, mc_name))
-                auth_needed = False
-
-        if auth_needed:
+        loaded = self._load_grok_creds()
+        if not loaded:
+            loaded = self._load_node_creds()
+        if not loaded:
             # normal XT client
             creds = cache_client.get_creds(self.team_name)
             if creds:
@@ -80,6 +43,65 @@ class XTVault():
                 creds = self._get_creds_from_login(authentication, reason="cache not set")
                 cache_client.store_creds(self.team_name, creds)
 
+    def _load_grok_creds(self):
+        fn_keys = "keys.bin"
+        loaded = False
+
+        if not os.path.exists(fn_keys):
+            fn_keys = os.path.expanduser("~/.xt/teams/{}/keys.bin".format(self.team_name))
+
+        if os.path.exists(fn_keys):
+            # GROK server creds
+            creds = file_utils.read_text_file(fn_keys)  
+            self.apply_creds(creds)
+
+            fn_cert = os.path.join(os.path.dirname(fn_keys), "xt_cert.pem")
+            if os.path.exists(fn_cert):
+                cert = file_utils.read_text_file(fn_keys) 
+                self.keys["xt_server_cert"] = cert
+
+            console.diag("init_creds: using grok server 'keys.bin' file")
+            loaded = True
+
+        return loaded
+
+    def _load_node_creds(self):
+        loaded = False
+
+        sc = os.getenv("XT_STORE_CREDS")
+        sc = utils.base64_to_text(sc)
+
+        mc = os.getenv("XT_MONGO_CONN_STR")
+        mc = utils.base64_to_text(mc)
+        #print("init_cred: sc={}, mc={}".format(sc, mc))
+
+        if sc and mc:
+            # XT client on compute node 
+
+            # cleanup (for testing from client)
+            sc = sc.replace('\\"', '"')
+            mc = mc.replace('\\"', '"')
+            if mc.startswith('"'):
+                mc = mc[1:-1]
+
+            # print("sc=", sc)
+            sc_data = json.loads(sc)
+            store_name = sc_data["name"]
+            store_key = sc_data["key"]
+
+            # sample mc: mongodb://xt-sandbox-cosmos:kBOWLQrseZ
+            prefix, mc_rest = mc.split("://", 1)
+            mc_name, _ = mc_rest.split(":", 1)
+
+            # creds are limited in this case to just Store access [storage + mongo]
+            creds = json.dumps({store_name: store_key, mc_name: mc})
+            self.apply_creds(creds)
+
+            console.print("init_creds: using compute node ENV VAR settings for store={}, mongo={}".format(store_name, mc_name))
+            loaded = True
+
+        return loaded
+
     def _get_creds_from_login(self, authentication, reason=None):
 
         # use normal Key Value
@@ -120,10 +142,6 @@ class XTVault():
 
         # write all our creds to self.keys
         self.apply_creds(key_text)
-        #self.keys["user_principle_name"] = get_user_principle_name(token)
-        # self.keys["token"] = token
-        # self.keys["outer_token"] = outer_token
-        #self.keys["xt_client_cert"] = xt_client_cert
         self.keys["xt_server_cert"] = xt_server_cert
 
         # return creds as json string
@@ -132,8 +150,11 @@ class XTVault():
     def apply_creds(self, creds_text):
         self.keys = json.loads(creds_text)
 
-    def get_secret(self, id, prop_name):
+    def get_secret(self, id):
         # returned cached copy (only ever needs 1 roundtrip)
+        if not id in self.keys:
+            errors.creds_error("Missing key in memory vault: {}".format(id))
+
         return self.keys[id]
 
     def get_secret_live(self, id):
